@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
+# Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -35,29 +35,10 @@
 #  $makefile->libraries() ;
 #  $makefile->includes() ;
 #  $makefile->definitions() ;
-#  $makefile->compile_options() ;
 #  $makefile->sources('foo') ;
 #  $makefile->our_libs('foo') ;
 #  $makefile->sys_libs('foo') ;
 #
-# Typical directories in a autoconf vpath build (see includes()):
-#
-#    project <-- $(top_srcdir)        <-------+-+ "base_to_top"
-#       |                                     | |
-#       +----src  <-- readall() base        --+ | <---+ base()
-#       |     |                                 |     |
-#       |     +-- sub1  <-- path()              | ----+
-#       |                                       |
-#       +----bin                                |
-#       |                                       |
-#       +--build <-- $(top_builddir)            |  <-- $(top_builddir)
-#            |                                  |
-#            +-- src                          --+  <--+ base()
-#                 |                                   |
-#                 +-- sub1  <-- c++ cwd         ------+
-#
-# See also ConfigStatus.pm.
-#
 
 use strict ;
 use FileHandle ;
@@ -70,20 +51,19 @@ our $debug = 0 ;
 
 sub new
 {
-	my ( $classname , $path , $switches , $vars_in ) = @_ ;
+	my ( $classname , $path , $switches , $ro_vars ) = @_ ;
 	my %me = (
 		m_path => simplepath($path) ,
-		m_switches => $switches ,
-		m_vars => {} ,
 		m_depth => undef , # from readall()
 		m_lines => [] ,
+		m_vars => {} ,
+		m_switches => $switches ,
 		m_stack => [] ,
 	) ;
 	my $this = bless \%me , $classname ;
 	$this->read( $path ) ;
 	$this->parse( $path ) ;
-	$this->expand_all( $vars_in ) ;
-	$this->copy( $vars_in ) ;
+	$this->expand_all( $ro_vars ) ;
 	return $this ;
 }
 
@@ -94,23 +74,22 @@ sub path
 
 sub readall
 {
-	my ( $base_dir , $switches , $vars , $verbose , $verbose_prefix ) = @_ ;
+	my ( $base_dir , $switches , $vars , $verbose ) = @_ ;
 	my @makefiles = () ;
-	_readall_imp( \@makefiles , 0 , $base_dir , $switches , $vars , $verbose , $verbose_prefix ) ; # recursive
+	_readall_imp( \@makefiles , 0 , $base_dir , $switches , $vars , $verbose ) ; # recursive
 	return @makefiles ;
 }
 
 sub _readall_imp
 {
-	my ( $makefiles , $depth , $dir , $switches , $vars , $verbose , $verbose_prefix ) = @_ ;
-	$verbose_prefix ||= "" ;
+	my ( $makefiles , $depth , $dir , $switches , $vars , $verbose ) = @_ ;
 	my $m = new AutoMakeParser( "$dir/Makefile.am" , $switches , $vars ) ;
 	$m->{m_depth} = $depth ;
-	print "${verbose_prefix}makefile=[" , $m->path() , "] ($depth)\n" if $verbose ;
+	print "makefile=[" , $m->path() , "] ($depth)\n" if $verbose ;
 	push @$makefiles , $m ;
 	for my $subdir ( $m->value("SUBDIRS") )
 	{
-		_readall_imp( $makefiles , $depth+1 , "$dir/$subdir" , $switches , $vars , $verbose , $verbose_prefix ) ;
+		_readall_imp( $makefiles , $depth+1 , "$dir/$subdir" , $switches , $vars , $verbose ) ;
 	}
 }
 
@@ -122,11 +101,11 @@ sub depth
 	return $this->{m_depth} ;
 }
 
-sub base
+sub top
 {
 	# Returns the relative path up to the first readall()
-	# makefile. The returned value will be something like
-	# "../../../". See also includes().
+	# makefile, which might be different from $(top_srcdir).
+	# The returned value will be something like "../../../".
 	#
 	my ( $this ) = @_ ;
 	my $depth = $this->{m_depth} ;
@@ -180,15 +159,10 @@ sub sys_libs
 {
 	my ( $this , $program ) = @_ ;
 	( my $prefix = $program ) =~ s/[-.]/_/g ;
-	my @a =
-		map { s/-l// ; $_ }
-		grep { m/^-l/ }
-		$this->value( "LIBS" ) ;
-	my @b =
+	return
 		map { s/-l// ; $_ }
 		grep { m/^-l/ }
 		$this->value( "${prefix}_LDADD" ) ;
-	return ( @a , @b ) ;
 }
 
 sub sources
@@ -200,51 +174,11 @@ sub sources
 		$this->value( "${prefix}_SOURCES" ) ;
 }
 
-sub link_options
-{
-	my ( $this ) = @_ ;
-	my @a = $this->value( "AM_LDFLAGS" ) ;
-	my @b = $this->value( "LDFLAGS" ) ;
-	my @options = ( @a , @b ) ;
-	return wantarray ? @options : join(" ",@options) ;
-}
-
-sub compile_options
-{
-	my ( $this ) = @_ ;
-	my @a = $this->_compile_options_imp( "AM_CPPFLAGS" , $this->{m_vars} ) ;
-	my @b = $this->_compile_options_imp( "CXXFLAGS" , $this->{m_vars} ) ;
-	my @options = ( @a , @b ) ;
-	return wantarray ? @options : join(" ",@options) ;
-}
-
-sub _compile_options_imp
-{
-	my ( $this , $var , $vars ) = @_ ;
-	$vars ||= $this->{m_vars} ;
-	my $s = protect_quoted_spaces( simple_spaces( $vars->{$var} ) ) ;
-	$s =~ s/-D /-D/g ;
-	$s =~ s/-I /-I/g ;
-	return
-		map { s/\t/ /g ; $_ }
-		grep { !m/-I/ }
-		grep { !m/-D/ }
-		split( " " , $s ) ;
-}
-
 sub definitions
 {
-	my ( $this ) = @_ ;
-	my @a = $this->_definitions_imp( "AM_CPPFLAGS" , $this->{m_vars} ) ;
-	my @b = $this->_definitions_imp( "CXXFLAGS" , $this->{m_vars} ) ;
-	my @defs = ( @a , @b ) ;
-	return wantarray ? @defs : join(" ",@defs) ;
-}
-
-sub _definitions_imp
-{
-	my ( $this , $var , $vars ) = @_ ;
-	my $s = protect_quoted_spaces( simple_spaces( $vars->{$var} ) ) ;
+	my ( $this , $am_cppflags ) = @_ ;
+	$am_cppflags ||= "AM_CPPFLAGS" ;
+	my $s = protect_quoted_spaces( simple_spaces( $this->{m_vars}->{$am_cppflags} ) ) ;
 	$s =~ s/-D /-D/g ;
 	return
 		map { s/\t/ /g ; $_ }
@@ -255,47 +189,34 @@ sub _definitions_imp
 
 sub includes
 {
-	# Returns a list of include directories derived from the
-	# AM_CPPFLAGS and CXXFLAGS macros. The returned list also
-	# optionally starts with the autoconf header directory,
-	# obtained by expanding top_srcdir.
+	# Returns a list of include directories, so for example
+	# "-I$(top_srcdir)/one/two -I$(top_srcdir)/three"
+	# with the 'top_srcdir' variable defined as "." gives
+	# ("./one/two","./three").
 	#
-	# Include paths need to vary through the source tree,
-	# so a 'base' parameter is provided here which is used
-	# as a prefix for all relative paths from the AM_CPPFLAGS
-	# and CXXFLAGS expansions and as a suffix for the
-	# autoconf header directory.
+	# However, since the 'top_srcdir' expansion is fixed, and the
+	# relative include paths need to vary through the source
+	# tree, a prefix parameter ('top') can be passed in here.
+	# So then "-I$(top_srcdir)/one/two" becomes "<top>/./one/two".
 	#
-	# For example, if CXXFLAGS is "-I$(top_srcdir)/src/sub"
-	# and top_srcdir is "." then includes(base()) will
-	# will yield ("./..",".././src/sub") for one makefile and
-	# ("./../..","../.././src/sub") for another.
+	# The 'top' parameter can be from top() as long as readall()
+	# started at the 'top_srcdir' directory and the 'top_srcdir'
+	# variable is defined as ".". Otherwise, a simple approach
+	# is to still use top() for the 'top' parameter but define
+	# the 'top_srcdir' variable as the difference between the
+	# readall() base and the actual 'top_srcdir' directory.
 	#
-	# In practice the value for top_srcdir should be carefully
-	# chosen as some "base-to-top" relative path that makes things
-	# work correctly if readall() was not based at top_srcdir
-	# or when targeting vpath builds. See above.
-	#
-	my ( $this , $base , $full_paths , $no_autoconf_dir ) = @_ ;
-	$base ||= "" ;
-	my $add_autoconf_dir = !$no_autoconf_dir ;
-	my $autoconf_dir = simplepath( join( "/" , $this->value("top_srcdir") , $base ) ) ;
-	my @a = $this->_includes_imp( $base , "AM_CPPFLAGS" , $this->{m_vars} , $full_paths ) ;
-	my @b = $this->_includes_imp( $base , "CXXFLAGS" , $this->{m_vars} , $full_paths ) ;
-	my @c = ( $autoconf_dir && $add_autoconf_dir ) ? ( $autoconf_dir ) : () ;
-	my @incs = ( @c , @a , @b ) ;
-	return wantarray ? @incs : join(" ",@incs) ;
-}
-
-sub _includes_imp
-{
-	my ( $this , $base , $var , $vars , $full_paths ) = @_ ;
-	my $s = protect_quoted_spaces( simple_spaces( $vars->{$var} ) ) ;
+	my ( $this , $top , $am_cppflags , $extra_includes , $full_paths ) = @_ ;
+	$top ||= "" ;
+	$am_cppflags ||= "AM_CPPFLAGS" ;
+	$extra_includes ||= "" ;
+	my $cppflags = join( " " , $extra_includes , $this->{m_vars}->{$am_cppflags} ) ;
+	my $s = protect_quoted_spaces( simple_spaces( $cppflags ) ) ;
 	$s =~ s/-I /-I/g ;
 	return
 		map { $full_paths?$this->fullpath($_):$_ }
 		map { simplepath($_) }
-		map { my $p=$_ ; ($base&&($p!~m;^/;))?join("/",$base,$p):$p }
+		map { $top?join("/",$top,$_):$_ }
 		map { s/\t/ /g ; $_ }
 		map { s:-I:: ; $_ } grep { m/-I\S+/ }
 		split( " " , $s ) ;
@@ -317,7 +238,7 @@ sub simplepath
 	for my $x ( @split )
 	{
 		next if( $x eq "" || $x eq "." ) ;
-		if( $x eq ".." && scalar(@out) && $out[-1] ne ".." )
+		if( $x eq ".." && scalar(@out) && @out[-1] ne ".." )
 		{
 			pop @out ;
 		}
@@ -398,19 +319,6 @@ sub add
 	push @{$this->{m_lines}} , $line ;
 }
 
-sub copy
-{
-	my ( $this , $vars_in ) = @_ ;
-	return if !defined($vars_in) ;
-	for my $k ( keys %$vars_in )
-	{
-		if( !exists($this->{m_vars}->{$k}) )
-		{
-			$this->{m_vars}->{$k} = $vars_in->{$k} ;
-		}
-	}
-}
-
 sub read
 {
 	my ( $this , $path ) = @_ ;
@@ -449,7 +357,7 @@ sub parse
 				last ; # (new)
 			}
 		}
-		debug_( "$$this{m_path}($n): " , $line ) if ( $this->enabled() && ( $line =~ m/\S/ ) && ( $line !~ m/^#/ ) ) ;
+		debug_( "$$this{m_path}($n): " , $this->enabled() ? $line : "..." ) ;
 	}
 	for my $k ( sort keys %{$this->{m_vars}} )
 	{
@@ -561,9 +469,7 @@ sub do_assign_more
 
 sub debug_
 {
-	my $line = join( " " , @_ ) ;
-	$line =~ s/ *\t */ /g ;
-	print $line , "\n" if $debug ;
+	print @_ , "\n" if $debug ;
 }
 
 1 ;
