@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright (C) 2001-2022 Graeme Walker <graeme_walker@users.sourceforge.net>
+# Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@
 #  winbuild::clean_cmake_cache_files(...) ;
 #  winbuild::deltree(...) ;
 #  winbuild::run_msbuild(...) ;
-#  winbuild::translate(...) ;
 #  winbuild::create_touchfile(...) ;
 #  winbuild::read_makefiles(...) ;
 #  winbuild::read_makefiles_imp(...) ;
@@ -146,9 +145,7 @@ sub default_touchfile
 
 sub _path_dirs
 {
-	my $path = $ENV{PATH} ;
-	my $sep = ( $path =~ m/;/ ) ? ";" : ":" ;
-	return split( $sep , $path ) ;
+	return split( ";" , $ENV{PATH} ) ;
 }
 
 sub _sanepath
@@ -178,7 +175,6 @@ sub _find_under
 	my $result ;
 	for my $dir ( map {_sanepath($_)} @dirs )
 	{
-		next if !$dir ;
 		my @find_list = () ;
 		File::Find::find( sub { push @find_list , $File::Find::name if lc($_) eq $fname } , $dir ) ;
 		if( @find_list ) { $result = Cwd::realpath($find_list[0]) ; last }
@@ -330,19 +326,6 @@ sub run_msbuild
 	die unless $rc == 0 ;
 }
 
-sub translate
-{
-	my ( $arch , $qt_dirs , $xx_XX , $xx ) = @_ ;
-	my $dir = $qt_dirs->{$arch} ;
-	$dir = File::Basename::dirname( $dir ) ;
-	$dir = File::Basename::dirname( $dir ) ;
-	$dir = File::Basename::dirname( $dir ) ;
-	my $tool = join( "/" , $dir , "bin" , "lrelease.exe" ) ;
-	my $rc = system( $tool , "src/gui/emailrelay_tr.$xx_XX.ts" , "-qm" , "src/gui/emailrelay.$xx.qm" ) ;
-	print "lrelease-exit=[$rc]\n" ;
-	die unless $rc == 0 ;
-}
-
 sub create_touchfile
 {
 	my ( $touchfile ) = @_ ;
@@ -353,8 +336,7 @@ sub create_touchfile
 sub read_makefiles
 {
 	my ( $switches , $vars ) = @_ ;
-	my $verbose = 1 ;
-	return AutoMakeParser::readall( "." , $switches , $vars , $verbose ) ;
+	return AutoMakeParser::readall( "." , $switches , $vars , 1 ) ;
 }
 
 sub cache_value
