@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -36,12 +36,10 @@ namespace GAuth
 	class SaslServerImp ;
 }
 
-/// \class GAuth::SaslServer
+//| \class GAuth::SaslServer
 /// An interface for implementing the server-side SASL challenge/response
-/// concept. Third-party libraries could be plumbed in at this interface in order
-/// to get support for more exotic authentication mechanisms. In practice there is
-/// one derived class for basic authentication mechanisms using a secrets file,
-/// and another for PAM.
+/// concept. In practice there is one derived class for basic authentication
+/// mechanisms using a secrets file, and another for PAM.
 ///
 /// Usage:
 /// \code
@@ -55,9 +53,7 @@ namespace GAuth
 ///     sasl.apply(peer.initialResponse(),done) : sasl.initialChallenge() ;
 ///   while( !done )
 ///   {
-/// \endcode
-/// peer.send( challenge ) ;
-/// \code
+///     peer.send( challenge ) ;
 ///     string response = peer.receive() ;
 ///     challenge = sasl.apply( response , done ) ;
 ///   }
@@ -70,7 +66,7 @@ namespace GAuth
 class GAuth::SaslServer
 {
 public:
-	virtual ~SaslServer() ;
+	virtual ~SaslServer() = default ;
 		///< Destructor.
 
 	virtual bool requiresEncryption() const = 0 ;
