@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -33,7 +33,7 @@ namespace GAuth
 	class SecretsFile ;
 }
 
-/// \class GAuth::Secrets
+//| \class GAuth::Secrets
 /// A simple interface for a store of secrets used in authentication.
 /// The default implementation uses a flat file.
 ///
@@ -65,30 +65,30 @@ public:
 	Secrets() ;
 		///< Default constructor for an in-valid(), empty-path object.
 
-	virtual ~Secrets() ;
-		///< Destructor.
-
-	virtual bool valid() const override ;
+	bool valid() const override ;
 		///< Override from GAuth::Valid virtual base.
 
-	virtual Secret serverSecret( const std::string & mechanism , const std::string & id ) const override ;
+	Secret serverSecret( const std::string & mechanism , const std::string & id ) const override ;
 		///< Override from GAuth::SaslServerSecrets.
 
-	virtual bool contains( const std::string & mechanism ) const override ;
+	bool contains( const std::string & mechanism ) const override ;
 		///< Override from GAuth::SaslServerSecrets.
 
+public:
+	~Secrets() override ;
+	Secrets( const Secrets & ) = delete ;
+	Secrets( Secrets && ) = delete ;
+	void operator=( const Secrets & ) = delete ;
+	void operator=( Secrets && ) = delete ;
+
 private: // overrides
-	virtual std::string source() const override ; // Override from GAuth::SaslServerSecrets.
-	virtual Secret clientSecret( const std::string & mechanism ) const override ; // Override from GAuth::SaslClientSecrets.
-	virtual std::pair<std::string,std::string> serverTrust( const std::string & address_range ) const override ; // Override from GAuth::SaslServerSecrets.
-
-private:
-	Secrets( const Secrets & ) g__eq_delete ;
-	void operator=( const Secrets & ) g__eq_delete ;
+	std::string source() const override ; // Override from GAuth::SaslServerSecrets.
+	Secret clientSecret( const std::string & mechanism ) const override ; // Override from GAuth::SaslClientSecrets.
+	std::pair<std::string,std::string> serverTrust( const std::string & address_range ) const override ; // Override from GAuth::SaslServerSecrets.
 
 private:
 	std::string m_source ;
-	SecretsFile * m_imp ;
+	std::unique_ptr<SecretsFile> m_imp ;
 } ;
 
 #endif
