@@ -1,22 +1,22 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
-//
-// garg.cpp
-//
+///
+/// \file garg.cpp
+///
 
 #include "gdef.h"
 #include "garg.h"
@@ -31,7 +31,7 @@ bool G::Arg::m_first = true ;
 std::string G::Arg::m_v0 ;
 std::string G::Arg::m_cwd ;
 
-G::Arg::Arg( int argc , char *argv[] )
+G::Arg::Arg( int argc , char **argv )
 {
 	G_ASSERT( argc > 0 ) ;
 	G_ASSERT( argv != nullptr ) ;
@@ -52,9 +52,7 @@ G::Arg::Arg( const StringArray & args ) :
 }
 
 G::Arg::Arg()
-{
-	// now use parse()
-}
+= default ; // now use parse()
 
 void G::Arg::parse( HINSTANCE , const std::string & command_line_tail )
 {
@@ -62,20 +60,21 @@ void G::Arg::parse( HINSTANCE , const std::string & command_line_tail )
 	if( proc_exe.empty() ) throw Exception( "cannot determine the path of this executable" ) ;
 	m_array.clear() ;
 	m_array.push_back( proc_exe ) ;
-	parseCore( command_line_tail ) ;
+	parseImp( command_line_tail ) ;
 }
 
 void G::Arg::parse( const std::string & command_line )
 {
 	G_ASSERT( !command_line.empty() ) ;
 	m_array.clear() ;
-	parseCore( command_line ) ;
+	parseImp( command_line ) ;
 }
 
 void G::Arg::reparse( const std::string & command_line_tail )
 {
-	while( m_array.size() > 1U ) m_array.pop_back() ;
-	parseCore( command_line_tail ) ;
+	auto p = m_array.begin() ;
+	m_array.erase( ++p , m_array.end() ) ;
+	parseImp( command_line_tail ) ;
 }
 
 std::string G::Arg::v0()
@@ -91,78 +90,106 @@ G::StringArray G::Arg::array( unsigned int shift ) const
 	return result ;
 }
 
-bool G::Arg::contains( const std::string & option , size_type option_args , bool cs ) const
+bool G::Arg::contains( const std::string & option , std::size_t option_args , bool cs ) const
 {
-	return find( cs , option , option_args , nullptr ) ;
+	return find( cs , option , option_args , nullptr ) != 0U ;
 }
 
-bool G::Arg::find( bool cs , const std::string & option , size_type option_args , size_type * index_p ) const
+std::size_t G::Arg::count( const std::string & option )
 {
-	for( size_type i = 1 ; i < m_array.size() ; i++ ) // start from v[1]
+	return find( true , option , 0U , nullptr ) ;
+}
+
+std::size_t G::Arg::find( bool cs , const std::string & option , std::size_t option_args ,
+	std::size_t * index_p ) const
+{
+	std::size_t count = 0U ;
+	for( std::size_t i = 1U ; i < m_array.size() ; i++ ) // start from v[1]
 	{
-		if( match(cs,option,m_array[i]) && (i+option_args) < m_array.size() )
+		if( strmatch(cs,option,m_array[i]) && (i+option_args) < m_array.size() )
 		{
+			count++ ;
 			if( index_p != nullptr )
 				*index_p = i ;
-			return true ;
+			i += option_args ;
 		}
 	}
-	return false ;
+	return count ;
 }
 
-bool G::Arg::match( bool cs , const std::string & s1 , const std::string & s2 )
+std::size_t G::Arg::match( const std::string & prefix ) const
+{
+	for( std::size_t i = 1U ; i < m_array.size() ; i++ )
+	{
+		if( G::Str::headMatch(m_array[i],prefix) )
+		{
+			return i ;
+		}
+	}
+	return 0U ;
+}
+
+bool G::Arg::strmatch( bool cs , const std::string & s1 , const std::string & s2 )
 {
 	return cs ? (s1==s2) : (Str::upper(s1)==Str::upper(s2)) ;
 }
 
-bool G::Arg::remove( const std::string & option , size_type option_args )
+bool G::Arg::remove( const std::string & option , std::size_t option_args )
 {
-	size_type i = 0U ;
+	std::size_t i = 0U ;
 	const bool found = find( true , option , option_args , &i ) ;
 	if( found )
 		removeAt( i , option_args ) ;
 	return found ;
 }
 
-void G::Arg::removeAt( size_type option_index , size_type option_args )
+std::string G::Arg::removeAt( std::size_t option_index , std::size_t option_args )
 {
-	G_ASSERT( option_index > 0U && option_index < m_array.size() ) ;
-	if( option_index > 0U && option_index < m_array.size() )
+	std::string value ;
+	if( option_index > 0U && (option_index+option_args) < m_array.size() )
 	{
-		StringArray::iterator p = m_array.begin() ;
-		for( size_type i = 0U ; i < option_index ; i++ ) ++p ; // (rather than cast)
+		value = v( option_index + (option_args?1U:0U) , std::string() ) ;
+		auto p = m_array.begin() ;
+		for( std::size_t i = 0U ; i < option_index ; i++ ) ++p ; // (rather than cast)
 		p = m_array.erase( p ) ;
-		for( size_type i = 0U ; i < option_args && p != m_array.end() ; i++ )
+		for( std::size_t i = 0U ; i < option_args && p != m_array.end() ; i++ )
 			p = m_array.erase( p ) ;
 	}
+	return value ;
 }
 
-G::Arg::size_type G::Arg::index( const std::string & option , size_type option_args ) const
+std::size_t G::Arg::index( const std::string & option , std::size_t option_args ,
+	std::size_t default_ ) const
 {
-	size_type i = 0U ;
+	std::size_t i = 0U ;
 	const bool found = find( true , option , option_args , &i ) ;
-	return found ? i : 0U ;
+	return found ? i : default_ ;
 }
 
-G::Arg::size_type G::Arg::c() const
+std::size_t G::Arg::c() const
 {
 	return m_array.size() ;
 }
 
-std::string G::Arg::v( size_type i ) const
+std::string G::Arg::v( std::size_t i ) const
 {
 	G_ASSERT( i < m_array.size() ) ;
 	return m_array.at(i) ;
 }
 
+std::string G::Arg::v( std::size_t i , const std::string & default_ ) const
+{
+	return i < m_array.size() ? m_array.at(i) : default_ ;
+}
+
 std::string G::Arg::prefix() const
 {
-	G_ASSERT( m_array.size() > 0U ) ;
+	G_ASSERT( !m_array.empty() ) ;
 	Path path( m_array.at(0U) ) ;
 	return path.withoutExtension().basename() ;
 }
 
-const char * G::Arg::prefix( char * argv [] ) // noexcept
+const char * G::Arg::prefix( char ** argv ) noexcept
 {
 	const char * exe = argv[0] ;
 	const char * p1 = std::strrchr( exe , '/' ) ;
@@ -172,63 +199,14 @@ const char * G::Arg::prefix( char * argv [] ) // noexcept
 	return p1 > p2 ? p1 : p2 ;
 }
 
-void G::Arg::parseCore( const std::string & command_line )
+void G::Arg::parseImp( const std::string & command_line )
 {
-	std::string s( command_line ) ;
-	protect( s ) ;
-	Str::splitIntoTokens( s , m_array , " " ) ;
-	unprotect( m_array ) ;
-	dequote( m_array ) ;
-}
-
-void G::Arg::protect( std::string & s )
-{
-	// replace all quoted spaces with a replacement -- this is not
-	// great because it does not allow for escaped quotes etc, but
-	// the relevant operating system's command-line parsing is
-	// appallingly bad to start with
-	bool in_quote = false ;
-	const char quote = '"' ;
-	const char space = ' ' ;
-	const char replacement = '\0' ;
-	for( std::string::size_type pos = 0U ; pos < s.length() ; pos++ )
-	{
-		if( s.at(pos) == quote ) in_quote = ! in_quote ;
-		if( in_quote && s.at(pos) == space ) s[pos] = replacement ;
-	}
-}
-
-void G::Arg::unprotect( StringArray & array )
-{
-	// restore replacements to spaces
-	const char space = ' ' ;
-	const char replacement = '\0' ;
-	for( StringArray::iterator p = array.begin() ; p != array.end() ; ++p )
-	{
-		std::string & s = *p ;
-		Str::replaceAll( s , std::string(1U,replacement) , std::string(1U,space) ) ;
-	}
-}
-
-void G::Arg::dequote( StringArray & array )
-{
-	// remove quotes if first and last characters (or equivalent)
-	char qq = '\"' ;
-	for( StringArray::iterator p = array.begin() ; p != array.end() ; ++p )
-	{
-		std::string & s = *p ;
-		if( s.length() > 1U )
-		{
-			std::string::size_type start = s.at(0U) == qq ? 0U : s.find("=\"") ;
-			if( start != std::string::npos && s.at(start) != qq ) ++start ;
-			std::string::size_type end = s.at(s.length()-1U) == qq ? (s.length()-1U) : std::string::npos ;
-			if( start != std::string::npos && end != std::string::npos && start != end )
-			{
-				s.erase( end , 1U ) ; // first!
-				s.erase( start , 1U ) ;
-			}
-		}
-	}
+	string_view ws( " \t" ) ;
+	string_view nbws( "\0\0" , 2U ) ;
+	const char esc = '\\' ;
+	const char qq = '\"' ;
+	G::Str::splitIntoTokens( G::Str::dequote(command_line,qq,esc,ws,nbws) , m_array , ws , esc ) ;
+	G::Str::replace( m_array , '\0' , ' ' ) ;
 }
 
 std::string G::Arg::exe( bool do_throw )
@@ -257,4 +235,3 @@ std::string G::Arg::exe( bool do_throw )
 	}
 }
 
-/// \file garg.cpp
