@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -31,7 +31,7 @@ namespace G
 	class Cleanup ;
 }
 
-/// \class G::Cleanup
+//| \class G::Cleanup
 /// A static interface for registering cleanup functions that are called when
 /// the process terminates abnormally. On unix this relates to signals like
 /// SIGTERM, SIGINT etc.
@@ -40,11 +40,21 @@ class G::Cleanup
 {
 public:
 	G_EXCEPTION( Error , "cleanup error" ) ;
+	struct Block /// A RAII class to temporarily block signal delivery.
+	{
+		explicit Block( bool active = true ) noexcept ;
+		~Block() ;
+		bool m_active ;
+		Block( const Block & ) = delete ;
+		Block( Block && ) = delete ;
+		void operator=( const Block & ) = delete ;
+		void operator=( Block && ) = delete ;
+	} ;
 
 	static void init() ;
 		///< An optional early-initialisation function. May be called more than once.
 
-	static void add( void (*fn)(SignalSafe,const char*) , const char * arg ) ;
+	static void add( bool (*fn)(SignalSafe,const char*) , const char * arg ) ;
 		///< Adds the given handler to the list of handlers that are to be called
 		///< when the process terminates abnormally. The handler function must be
 		///< fully reentrant, hence the SignalSafe dummy parameter. The 'arg'
@@ -58,8 +68,31 @@ public:
 		///< might call exit(), but be careful to disable these exit handlers
 		///< before normal termination by calling atexit(false).
 
-private:
-	Cleanup() g__eq_delete ;
+	static void block() noexcept ;
+		///< Temporarily blocks signals until release()d. This should be used
+		///< before creating threads so that only the main thread does signal
+		///< handling.
+
+	static void release() noexcept ;
+		///< Releases block()ed signals.
+
+public:
+	Cleanup() = delete ;
 } ;
 
+inline
+G::Cleanup::Block::Block( bool active ) noexcept :
+	m_active(active)
+{
+	if( m_active )
+		Cleanup::block() ;
+}
+
+inline
+G::Cleanup::Block::~Block()
+{
+	if( m_active )
+		Cleanup::release() ;
+}
+
 #endif
