@@ -1,22 +1,22 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
-//
-// gcleanup_win32.cpp
-//
+///
+/// \file gcleanup_win32.cpp
+///
 
 #include "gdef.h"
 #include "gcleanup.h"
@@ -26,7 +26,7 @@ void G::Cleanup::init()
 	// no-op
 }
 
-void G::Cleanup::add( void (*)(SignalSafe,const char*) , const char * )
+void G::Cleanup::add( bool (*)(SignalSafe,const char*) , const char * )
 {
 	// not implemented
 }
@@ -36,4 +36,13 @@ void G::Cleanup::atexit( bool )
 	// not implemented
 }
 
-/// \file gcleanup_win32.cpp
+void G::Cleanup::block() noexcept
+{
+	// not implemented
+}
+
+void G::Cleanup::release() noexcept
+{
+	// not implemented
+}
+
