@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -37,7 +37,7 @@ namespace G
 	class DirectoryList ;
 }
 
-/// \class G::Directory
+//| \class G::Directory
 /// An encapsulation of a file system directory that works with
 /// G::DirectoryIterator.
 /// \see G::Path, G::FileSystem, G::File
@@ -57,22 +57,27 @@ public:
 	explicit Directory( const std::string & path ) ;
 		///< Constructor.
 
-	bool valid( bool for_creating_files = false ) const ;
-		///< Returns true if the object represents a valid directory.
+	int usable( bool for_creating_files = false ) const ;
+		///< Returns zero if the object represents a valid directory
+		///< with permissions that dont disallow reading of any
+		///< contained files. Returns a non-zero errno otherwise.
 		///<
 		///< Does additional checks if the 'for-creating-files'
 		///< parameter is true. But note that the answer is not
 		///< definitive -- file creation may fail, even if
-		///< valid() returns true. For a more accurate test use
+		///< usable() returns zero. For a more accurate test use
 		///< writeable().
 
-	bool writeable( std::string probe_filename = tmp() ) const ;
+	bool valid( bool for_creating_files = false ) const ;
+		///< Returns true iff usable() is zero.
+
+	bool writeable( const std::string & probe_filename = tmp() ) const ;
 		///< Tries to create and then delete an empty test file
 		///< in the directory. Returns true on success.
 		///< Precondition: valid()
 
 	Path path() const ;
-		///< Returns the directory's path.
+		///< Returns the directory's path, as passed in to the ctor.
 
 	static std::string tmp() ;
 		///< A convenience function for constructing a filename for
@@ -84,7 +89,7 @@ private:
 	Path m_path ;
 } ;
 
-/// \class G::DirectoryIterator
+//| \class G::DirectoryIterator
 /// A iterator that returns filenames in a directory.
 /// The iteration model is:
 /// \code
@@ -112,23 +117,27 @@ public:
 
 	std::string sizeString() const ;
 		///< Returns the file size as a decimal string. The value
-		///< may be more than 32 bits. See also class G::Number.
+		///< may be bigger than any integer type can hold.
 
 	Path filePath() const ;
 		///< Returns the path of the current item.
 
 	std::string fileName() const ;
-		///< Returns the name of the current item.
+		///< Returns the name of the current item. On Windows
+		///< any characters that cannot be represented in the
+		///< active code page are replaced by '?'.
+
+public:
+	DirectoryIterator( const DirectoryIterator & ) = delete ;
+	DirectoryIterator( DirectoryIterator && ) noexcept = default ;
+	void operator=( const DirectoryIterator & ) = delete ;
+	DirectoryIterator & operator=( DirectoryIterator && ) noexcept = default ;
 
 private:
-	DirectoryIterator( const DirectoryIterator & ) g__eq_delete ;
-	void operator=( const DirectoryIterator & ) g__eq_delete ;
-
-private:
-	unique_ptr<DirectoryIteratorImp> m_imp ;
+	std::unique_ptr<DirectoryIteratorImp> m_imp ;
 } ;
 
-/// \class G::DirectoryList
+//| \class G::DirectoryList
 /// A iterator similar to G::DirectoryIterator but doing all file
 /// i/o in one go. This is useful when temporarily adopting
 /// additional process privileges to read a directory.
@@ -138,7 +147,7 @@ class G::DirectoryList
 public:
 	struct Item /// A directory-entry item for G::DirectoryList.
 	{
-		bool m_is_dir ;
+		bool m_is_dir{false} ;
 		Path m_path ;
 		std::string m_name ;
 	} ;
@@ -166,7 +175,9 @@ public:
 		///< Returns the current path.
 
 	std::string fileName() const ;
-		///< Returns the current filename.
+		///< Returns the current filename. On Windows
+		///< any characters that cannot be represented in the
+		///< active code page are replaced by '?'.
 
 	void sort() ;
 		///< Sorts the files lexicographically.
@@ -179,8 +190,8 @@ private:
 	static bool compare( const Item & , const Item & ) ;
 
 private:
-	bool m_first ;
-	unsigned int m_index ;
+	bool m_first{true} ;
+	unsigned int m_index{0U} ;
 	std::vector<Item> m_list ;
 } ;
 
