@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -33,7 +33,7 @@ namespace G
 	class PamImp ;
 }
 
-/// \class G::Pam
+//| \class G::Pam
 /// A thin interface to the system PAM library, with two pure
 /// virtual methods that derived classes should implement: the
 /// converse() method supplies passwords etc. and delay()
@@ -71,7 +71,7 @@ public:
 		Error( const std::string & op , int pam_error ) ;
 		Error( const std::string & op , int pam_error , const char * ) ;
 	} ;
-	typedef std::vector<Item> ItemArray ;
+	using ItemArray = std::vector<Item> ;
 
 	Pam( const std::string & app , const std::string & user , bool silent ) ;
 		///< Constructor.
@@ -149,12 +149,14 @@ public:
 		///< initiating any new authentication while the timer
 		///< is running.
 
-private:
-	Pam( const Pam & ) g__eq_delete ;
-	void operator=( const Pam & ) g__eq_delete ;
+public:
+	Pam( const Pam & ) = delete ;
+	Pam( Pam && ) = delete ;
+	void operator=( const Pam & ) = delete ;
+	void operator=( Pam && ) = delete ;
 
 private:
-	unique_ptr<PamImp> m_imp ;
+	std::unique_ptr<PamImp> m_imp ;
 } ;
 
 inline
