@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -18,8 +18,8 @@
 /// \file gaddress.h
 ///
 
-#ifndef G_NET_ADDRESS__H
-#define G_NET_ADDRESS__H
+#ifndef G_NET_ADDRESS_H
+#define G_NET_ADDRESS_H
 
 #include "gdef.h"
 #include "gstrings.h"
@@ -33,11 +33,9 @@ namespace GNet
 	class Address6 ;
 	class AddressStorage ;
 	class AddressStorageImp ;
-	int inet_pton_imp( int f , const char * p , void * result ) ;
-	const char * inet_ntop_imp( int f , void * ap , char * buffer , size_t n ) ;
 }
 
-/// \class GNet::Address
+//| \class GNet::Address
 /// The GNet::Address class encapsulates a TCP/UDP transport address. The
 /// address is exposed as a 'sockaddr' structure for low-level socket
 /// operations.
@@ -54,11 +52,11 @@ namespace GNet
 class GNet::Address
 {
 public:
-	g__enum(Family)
+	enum class Family
 	{
 		ipv4 ,
 		ipv6
-	} ; g__enum_end(Family) ;
+	} ;
 
 	G_EXCEPTION( Error , "address error" ) ;
 	G_EXCEPTION( BadString , "invalid address" ) ;
@@ -68,6 +66,10 @@ public:
 		///< Returns true if the implementation supports the given address family,
 		///< either ipv4 or ipv6.
 
+	static bool supports( int af , int dummy ) ;
+		///< Returns true if the implementation supports the given address family,
+		///< either AF_INET or AF_INET6.
+
 	Address( const Address & ) ;
 		///< Copy constructor.
 
@@ -78,9 +80,14 @@ public:
 		///< Constructor using a given sockaddr. Throws an exception if an
 		///< invalid structure. See also: validData()
 
+	Address( const sockaddr * addr , socklen_t len , bool fixup ) ;
+		///< An overload that conditionally applies the bsd ipv6 scope-id
+		///< fixup.
+
 	Address( Family , unsigned int port ) ;
 		///< Constructor for a wildcard INADDR_ANY address with the given port
 		///< number. Throws an exception if an invalid port number.
+		///< Postcondition: isAny()
 		/// \see validPort()
 
 	explicit Address( const std::string & display_string ) ;
@@ -91,14 +98,18 @@ public:
 		///< Constructor taking an ip-address and a port number. Throws an
 		///< exception if an invalid string.
 
-	#if GCONFIG_HAVE_CXX_MOVE
-	Address( Address && ) g__noexcept ;
-		// Move constructor.
-	#endif
+	Address( Address && ) noexcept ;
+		///< Move constructor.
 
 	~Address() ;
 		///< Destructor.
 
+	Address & operator=( const Address & ) ;
+		///< Assignment operator.
+
+	Address & operator=( Address && ) noexcept ;
+		///< Move assignment operator.
+
 	static Address defaultAddress() ;
 		///< Returns a default address, being the IPv4 wildcard address
 		///< with a zero port number.
@@ -117,7 +128,7 @@ public:
 	socklen_t length() const;
 		///< Returns the size of the sockaddr address. See address().
 
-	std::string displayString() const ;
+	std::string displayString( bool with_scope_id = false ) const ;
 		///< Returns a string which represents the transport address for
 		///< debugging and diagnostics purposes.
 
@@ -141,10 +152,6 @@ public:
 	int af() const ;
 		///< Returns the address family number, AF_INET or AFINET6.
 
-	unsigned long scopeId( unsigned long default_ = 0UL ) const ;
-		///< Returns the scope-id. Returns the default if scope-ids are not
-		///< supported by the underlying address type.
-
 	static bool validPort( unsigned int n ) ;
 		///< Returns true if the port number is within the valid range. This
 		///< can be used to avoid exceptions from the relevant constructors.
@@ -166,10 +173,25 @@ public:
 		///< Returns true if the two addresses have the same host part
 		///< (ie. the network address, ignoring the port number).
 
-	void setPort( unsigned int port ) ;
+	Address & setPort( unsigned int port ) ;
 		///< Sets the port number. Throws an exception if an invalid
 		///< port number (ie. too big).
 
+	bool setZone( const std::string & ) ;
+		///< Sets the zone. The parameter is normally a decimal string
+		///< representation of the zone-id, aka scope-id (eg. "1"), but
+		///< if not numeric then it is treated as an interface name
+		///< which is mapped to a zone-id by if_nametoindex(3). Returns
+		///< false on error. Returns true if zones are not used by the
+		///< address family.
+
+	Address & setScopeId( unsigned long ) ;
+		///< Sets the scope-id.
+
+	unsigned long scopeId( unsigned long default_ = 0UL ) const ;
+		///< Returns the scope-id. Returns the default if scope-ids are not
+		///< supported by the underlying address type.
+
 	G::StringArray wildcards() const ;
 		///< Returns an ordered list of wildcard strings that match this
 		///< address. The fully-address-specific string (eg. "192.168.0.1")
@@ -183,14 +205,28 @@ public:
 	bool isLoopback() const ;
 		///< Returns true if this is a loopback address.
 
+	bool isLinkLocal() const ;
+		///< Returns true if this is a link-local address.
+
+	bool isUniqueLocal() const ;
+		///< Returns true if this is a locally administered address.
+
+	bool isAny() const ;
+		///< Returns true if this is the address family's 'any' address.
+
 	bool isLocal( std::string & reason ) const ;
 		///< Returns true if this seems to be a 'local' address, ie. an
 		///< address that is inherently more trusted. Returns an
 		///< explanation by reference otherwise.
 
-	bool isPrivate() const ;
-		///< Returns true if this seems to be a 'private' address,
-		///< as in RFC-1918 for IPv4.
+	bool is4() const ;
+		///< Returns true if family() is ipv4.
+
+	bool is6() const ;
+		///< Returns true if family() is ipv6.
+
+	bool same( const Address & , bool with_scope ) const ;
+		///< Comparison function.
 
 	bool operator==( const Address & ) const ;
 		///< Comparison operator.
@@ -198,34 +234,26 @@ public:
 	bool operator!=( const Address & ) const ;
 		///< Comparison operator.
 
-	void swap( Address & other ) g__noexcept ;
+	void swap( Address & other ) noexcept ;
 		///< Swaps this with other.
 
-	Address & operator=( const Address & addr ) ;
-		///< Assignment operator.
-
-	#if GCONFIG_HAVE_CXX_MOVE
-	Address & operator=( Address && ) g__noexcept ;
-		// Move assigmnemt operator.
-	#endif
-
 private:
 	Address( Family , unsigned int , int ) ; // loopback()
 
 private:
-	unique_ptr<Address4> m_4imp ;
-	unique_ptr<Address6> m_6imp ;
+	std::unique_ptr<Address4> m_4imp ;
+	std::unique_ptr<Address6> m_6imp ;
 } ;
 
 namespace GNet
 {
-	inline void swap( Address & a , Address & b ) g__noexcept
+	inline void swap( Address & a , Address & b ) noexcept
 	{
 		a.swap(b) ;
 	}
 }
 
-/// \class GNet::AddressStorage
+//| \class GNet::AddressStorage
 /// A helper class for calling accept(), getsockname() and getpeername()
 /// and hiding the definition of sockaddr_storage.
 ///
@@ -253,12 +281,14 @@ public:
 	socklen_t n() const ;
 		///< Returns the length, typically modified via p2().
 
-private:
-	AddressStorage( const AddressStorage & ) g__eq_delete ;
-	void operator=( const AddressStorage & ) g__eq_delete ;
+public:
+	AddressStorage( const AddressStorage & ) = delete ;
+	AddressStorage( AddressStorage && ) = delete ;
+	void operator=( const AddressStorage & ) = delete ;
+	void operator=( AddressStorage && ) = delete ;
 
 private:
-	unique_ptr<AddressStorageImp> m_imp ;
+	std::unique_ptr<AddressStorageImp> m_imp ;
 } ;
 
 #endif
