@@ -1,22 +1,22 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
-//
-// glinebuffer.cpp
-//
+///
+/// \file glinebuffer.cpp
+///
 
 #include "gdef.h"
 #include "glinebuffer.h"
@@ -25,7 +25,7 @@
 #include "gassert.h"
 #include <algorithm>
 
-GNet::LineBuffer::LineBuffer( LineBufferConfig config ) :
+GNet::LineBuffer::LineBuffer( const LineBufferConfig & config ) :
 	m_auto(config.eol().empty()) ,
 	m_eol(config.eol()) ,
 	m_warn_limit(config.warn()) ,
@@ -44,11 +44,15 @@ void GNet::LineBuffer::clear()
 	if( !transparent() )
 		m_expect = 0U ;
 
-	G_ASSERT( m_in.size() == 0U ) ;
-	G_ASSERT( !more(true) ) ;
+	G_ASSERT( m_in.empty() && empty() ) ;
 }
 
-void GNet::LineBuffer::add( const char * data , size_t size )
+bool GNet::LineBuffer::empty() const
+{
+	return state().empty() ;
+}
+
+void GNet::LineBuffer::add( const char * data , std::size_t size )
 {
 	m_in.append( data , size ) ;
 }
@@ -58,9 +62,10 @@ void GNet::LineBuffer::add( const std::string & s )
 	m_in.append( s ) ;
 }
 
-void GNet::LineBuffer::extensionStart( const char * data , size_t size )
+void GNet::LineBuffer::extensionStart( const char * data , std::size_t size )
 {
-	m_in.extend( data , size ) ;
+	if( data )
+		m_in.extend( data , size ) ;
 }
 
 void GNet::LineBuffer::extensionEnd()
@@ -72,8 +77,8 @@ void GNet::LineBuffer::extensionEnd()
 bool GNet::LineBuffer::more( bool fragments )
 {
 	G_ASSERT( m_pos <= m_in.size() ) ;
-	const size_t npos = std::string::npos ;
-	size_t pos = 0U ;
+	const std::size_t npos = std::string::npos ;
+	std::size_t pos = 0U ;
 
 	if( m_pos == m_in.size() )
 	{
@@ -98,7 +103,7 @@ bool GNet::LineBuffer::more( bool fragments )
 			// not all expected, return the available fragment
 			//
 			G_ASSERT( m_in.size() > m_pos ) ;
-			size_t n = m_in.size() - m_pos ;
+			std::size_t n = m_in.size() - m_pos ;
 			output( n , 0U ) ;
 			if( !transparent() ) m_expect -= n ;
 			return true ;
@@ -139,7 +144,7 @@ bool GNet::LineBuffer::more( bool fragments )
 	}
 }
 
-bool GNet::LineBuffer::trivial( size_t pos ) const
+bool GNet::LineBuffer::trivial( std::size_t pos ) const
 {
 	pos = pos == std::string::npos ? m_in.size() : pos ;
 	return ( pos - m_pos ) < m_fmin ;
@@ -147,10 +152,10 @@ bool GNet::LineBuffer::trivial( size_t pos ) const
 
 bool GNet::LineBuffer::detect()
 {
-	const size_t npos = std::string::npos ;
+	const std::size_t npos = std::string::npos ;
 	if( m_auto )
 	{
-		size_t pos = m_in.find( '\n' ) ;
+		std::size_t pos = m_in.find( '\n' ) ;
 		if( pos != npos )
 		{
 			if( pos > 0U && m_in.at(pos-1U) == '\r' )
@@ -163,7 +168,7 @@ bool GNet::LineBuffer::detect()
 	return !m_eol.empty() ;
 }
 
-void GNet::LineBuffer::expect( size_t n )
+void GNet::LineBuffer::expect( std::size_t n )
 {
 	m_expect = n ;
 }
@@ -178,7 +183,7 @@ std::string GNet::LineBuffer::eol() const
 	return m_eol ;
 }
 
-void GNet::LineBuffer::output( size_t size , size_t eolsize , bool force_next_is_start_of_line )
+void GNet::LineBuffer::output( std::size_t size , std::size_t eolsize , bool force_next_is_start_of_line )
 {
 	G_ASSERT( (size+eolsize) != 0U ) ;
 	m_pos += m_out.set( m_in , m_pos , size , eolsize ) ;
@@ -209,17 +214,10 @@ GNet::LineBufferState GNet::LineBuffer::state() const
 
 // ==
 
-GNet::LineBuffer::Output::Output() :
-	m_first(true) ,
-	m_data(nullptr) ,
-	m_size(0U) ,
-	m_eolsize(0U) ,
-	m_linesize(0U) ,
-	m_c0('\0')
-{
-}
+GNet::LineBuffer::Output::Output()
+= default;
 
-size_t GNet::LineBuffer::Output::set( LineStore & in , size_t pos , size_t size , size_t eolsize )
+std::size_t GNet::LineBuffer::Output::set( LineStore & in , std::size_t pos , std::size_t size , std::size_t eolsize )
 {
 	bool start = m_first || m_eolsize != 0U ; // ie. wrt previous line's eolsize
 	m_first = false ;
@@ -235,17 +233,18 @@ size_t GNet::LineBuffer::Output::set( LineStore & in , size_t pos , size_t size
 
 // ==
 
-GNet::LineBufferConfig::LineBufferConfig( const std::string & eol , size_t warn , size_t fmin , size_t expect ) :
-	m_eol(eol) ,
-	m_warn(warn) ,
-	m_fmin(fmin) ,
-	m_expect(expect)
+GNet::LineBufferConfig::LineBufferConfig( const std::string & eol , std::size_t warn ,
+	std::size_t fmin , std::size_t expect ) :
+		m_eol(eol) ,
+		m_warn(warn) ,
+		m_fmin(fmin) ,
+		m_expect(expect)
 {
 }
 
 GNet::LineBufferConfig GNet::LineBufferConfig::transparent()
 {
-	const size_t inf = ~(size_t(0)) ;
+	const std::size_t inf = ~(std::size_t(0)) ;
 	//G_ASSERT( (inf+1U) == 0U ) ;
 	return LineBufferConfig( std::string(1U,'\n') , 0U , 0U , inf ) ;
 }
@@ -279,4 +278,3 @@ GNet::LineBufferConfig GNet::LineBufferConfig::http()
 {
 	return crlf() ;
 }
-/// \file glinebuffer.cpp
