@@ -1,22 +1,22 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
-//
-// gtimer.cpp
-//
+///
+/// \file gtimer.cpp
+///
 
 #include "gdef.h"
 #include "gtimer.h"
@@ -27,7 +27,7 @@
 #include <algorithm>
 
 GNet::TimerBase::TimerBase( ExceptionSink es ) :
-	m_time(0)
+	m_time(G::TimerTime::zero())
 {
 	TimerList::instance().add( *this , es ) ;
 }
@@ -36,15 +36,15 @@ GNet::TimerBase::~TimerBase()
 {
 	try
 	{
-		if( TimerList::instance(TimerList::NoThrow()) != nullptr )
-			TimerList::instance().remove( *this ) ;
+		if( TimerList::ptr() != nullptr )
+			TimerList::ptr()->remove( *this ) ;
 	}
 	catch(...) // dtor
 	{
 	}
 }
 
-bool GNet::TimerBase::expired( G::EpochTime & now ) const
+bool GNet::TimerBase::expired( G::TimerTime & now ) const
 {
 	if( !active() )
 	{
@@ -56,52 +56,60 @@ bool GNet::TimerBase::expired( G::EpochTime & now ) const
 	}
 	else
 	{
-		if( now.s == 0 ) now = G::DateTime::now() ;
+		if( now == G::TimerTime::zero() ) now = G::TimerTime::now() ; // lazy optimisation
 		return m_time <= now ;
 	}
 }
 
 void GNet::TimerBase::startTimer( unsigned int time , unsigned int time_us )
 {
-	m_time = (time==0U && time_us==0U) ? history() : ( G::DateTime::now() + G::TimeInterval(time,time_us) ) ;
-	TimerList::instance().updateOnStart( *this ) ;
+	m_time = (time==0U && time_us==0U) ? history() : ( G::TimerTime::now() + G::TimeInterval(time,time_us) ) ;
+	TimerList::instance().updateOnStart( *this ) ; // adjust()
 }
 
-G::EpochTime GNet::TimerBase::history()
+void GNet::TimerBase::startTimer( const G::TimeInterval & i )
 {
-	return G::EpochTime( 1 ) ;
+	m_time = i == G::TimeInterval(0U) ? history() : ( G::TimerTime::now() + i ) ;
+	TimerList::instance().updateOnStart( *this ) ; // adjust()
+}
+
+G::TimerTime GNet::TimerBase::history()
+{
+	// base for an arbitrary historical one-second non-zero epoch
+	// time interval used for all immediate() timers
+	return G::TimerTime::zero() + G::TimeInterval( 1U ) ;
 }
 
 bool GNet::TimerBase::immediate() const
 {
-	return m_time.s == 1 ; // history() with any adjust()ment
+	return m_time.sameSecond( history() ) ; // history() ignoring any adjust()ment
 }
 
 void GNet::TimerBase::adjust( unsigned int us )
 {
-	G_ASSERT( m_time.s == 1 ) ;
-	m_time.us = std::min( 999999U , us ) ;
+	m_time = history() + G::TimeInterval( 0U , std::min( 999999U , us ) ) ;
+	G_ASSERT( immediate() ) ;
 }
 
 void GNet::TimerBase::cancelTimer()
 {
 	if( active() )
 	{
-		m_time = G::EpochTime(0) ;
+		m_time = G::TimerTime::zero() ;
 		TimerList::instance().updateOnCancel( *this ) ;
 	}
+	G_ASSERT( !active() ) ;
 }
 
 void GNet::TimerBase::doTimeout()
 {
 	G_ASSERT( active() ) ;
-	m_time = G::EpochTime(0) ;
+	m_time = G::TimerTime::zero() ;
 	onTimeout() ;
 }
 
-G::EpochTime GNet::TimerBase::t() const
+G::TimerTime GNet::TimerBase::t() const
 {
 	return m_time ;
 }
 
-/// \file gtimer.cpp
