@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -18,8 +18,8 @@
 /// \file gnewmessage.h
 ///
 
-#ifndef G_SMTP_NEW_MESSAGE__H
-#define G_SMTP_NEW_MESSAGE__H
+#ifndef G_SMTP_NEW_MESSAGE_H
+#define G_SMTP_NEW_MESSAGE_H
 
 #include "gdef.h"
 
@@ -29,7 +29,7 @@ namespace GSmtp
 	class MessageStoreImp ;
 }
 
-/// \class GSmtp::NewMessage
+//| \class GSmtp::NewMessage
 /// An abstract class to allow the creation of a new message in
 /// the message store.
 /// \see GSmtp::MessageStore
@@ -40,7 +40,7 @@ public:
 	virtual void addTo( const std::string & to , bool local ) = 0 ;
 		///< Adds a 'to' address.
 
-	virtual bool addText( const char * , size_t ) = 0 ;
+	virtual bool addText( const char * , std::size_t ) = 0 ;
 		///< Adds a line of content, typically ending with CR-LF.
 		///< Returns false on overflow.
 
@@ -63,7 +63,7 @@ public:
 		///< A convenience function that calls addText() taking
 		///< a string parameter and adding CR-LF.
 
-	virtual ~NewMessage() ;
+	virtual ~NewMessage() = default ;
 		///< Destructor. Rolls back any prepare()d storage
 		///< if un-commit()ed.
 } ;
