@@ -1,22 +1,22 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
-//
-// gprotocolmessagestore.cpp
-//
+///
+/// \file gprotocolmessagestore.cpp
+///
 
 #include "gdef.h"
 #include "gprotocolmessagestore.h"
@@ -25,9 +25,9 @@
 #include "gassert.h"
 #include "glog.h"
 
-GSmtp::ProtocolMessageStore::ProtocolMessageStore( MessageStore & store , unique_ptr<Filter> filter ) :
+GSmtp::ProtocolMessageStore::ProtocolMessageStore( MessageStore & store , std::unique_ptr<Filter> filter ) :
 	m_store(store) ,
-	m_filter(filter.release())
+	m_filter(std::move(filter))
 {
 	m_filter->doneSignal().connect( G::Slot::slot(*this,&ProtocolMessageStore::filterDone) ) ;
 }
@@ -58,10 +58,10 @@ bool GSmtp::ProtocolMessageStore::setFrom( const std::string & from , const std:
 	if( from.length() == 0U ) // => probably a failure notification message
 		G_WARNING( "GSmtp::ProtocolMessageStore: empty MAIL-FROM return path" ) ;
 
-	G_ASSERT( m_new_msg.get() == nullptr ) ;
+	G_ASSERT( m_new_msg == nullptr ) ;
 	clear() ; // just in case
 
-	m_new_msg.reset( m_store.newMessage(from,from_auth,"").release() ) ;
+	m_new_msg = m_store.newMessage( from , from_auth , "" ) ;
 
 	m_from = from ;
 	return true ; // accept any name
@@ -71,8 +71,8 @@ bool GSmtp::ProtocolMessageStore::addTo( const std::string & to , VerifierStatus
 {
 	G_DEBUG( "GSmtp::ProtocolMessageStore::addTo: " << to ) ;
 
-	G_ASSERT( m_new_msg.get() != nullptr ) ;
-	if( to.length() > 0U && m_new_msg.get() != nullptr )
+	G_ASSERT( m_new_msg != nullptr ) ;
+	if( to.length() > 0U && m_new_msg != nullptr )
 	{
 		if( !to_status.is_valid )
 		{
@@ -95,31 +95,32 @@ bool GSmtp::ProtocolMessageStore::addTo( const std::string & to , VerifierStatus
 void GSmtp::ProtocolMessageStore::addReceived( const std::string & received_line )
 {
 	G_DEBUG( "GSmtp::ProtocolMessageStore::addReceived" ) ;
-	if( m_new_msg.get() != nullptr )
+	if( m_new_msg != nullptr )
 		m_new_msg->addTextLine( received_line ) ;
 }
 
-bool GSmtp::ProtocolMessageStore::addText( const char * line_data , size_t line_size )
+bool GSmtp::ProtocolMessageStore::addText( const char * line_data , std::size_t line_size )
 {
-	G_ASSERT( m_new_msg.get() != nullptr ) ;
-	if( m_new_msg.get() == nullptr )
+	G_ASSERT( m_new_msg != nullptr ) ;
+	if( m_new_msg == nullptr )
 		return true ;
 	return m_new_msg->addText( line_data , line_size ) ;
 }
 
 std::string GSmtp::ProtocolMessageStore::from() const
 {
-	return m_new_msg.get() ? m_from : std::string() ;
+	return m_new_msg ? m_from : std::string() ;
 }
 
-void GSmtp::ProtocolMessageStore::process( const std::string & session_auth_id , const std::string & peer_socket_address ,
-	const std::string & peer_certificate )
+void GSmtp::ProtocolMessageStore::process( const std::string & session_auth_id ,
+	const std::string & peer_socket_address , const std::string & peer_certificate )
 {
 	try
 	{
-		G_DEBUG( "GSmtp::ProtocolMessageStore::process: \"" << session_auth_id << "\", \"" << peer_socket_address << "\"" ) ;
-		G_ASSERT( m_new_msg.get() != nullptr ) ;
-		if( m_new_msg.get() == nullptr )
+		G_DEBUG( "GSmtp::ProtocolMessageStore::process: \""
+			<< session_auth_id << "\", \"" << peer_socket_address << "\"" ) ;
+		G_ASSERT( m_new_msg != nullptr ) ;
+		if( m_new_msg == nullptr )
 			throw G::Exception( "internal error" ) ; // never gets here
 
 		// write ".new" envelope
@@ -133,7 +134,8 @@ void GSmtp::ProtocolMessageStore::process( const std::string & session_auth_id ,
 		{
 			// start the filter
 			if( !m_filter->simple() )
-				G_LOG( "GSmtp::ProtocolMessageStore::process: filter start: [" << m_filter->id() << "] [" << message_location << "]" ) ;
+				G_LOG( "GSmtp::ProtocolMessageStore::process: filter start: [" << m_filter->id() << "] "
+					<< "[" << message_location << "]" ) ;
 			m_filter->start( message_location ) ;
 		}
 	}
@@ -150,8 +152,8 @@ void GSmtp::ProtocolMessageStore::filterDone( int filter_result )
 	try
 	{
 		G_DEBUG( "GSmtp::ProtocolMessageStore::filterDone: " << filter_result ) ;
-		G_ASSERT( m_new_msg.get() != nullptr ) ;
-		if( m_new_msg.get() == nullptr )
+		G_ASSERT( m_new_msg != nullptr ) ;
+		if( m_new_msg == nullptr )
 			throw G::Exception( "internal error" ) ; // never gets here
 
 		const bool ok = filter_result == 0 ;
@@ -201,9 +203,8 @@ void GSmtp::ProtocolMessageStore::filterDone( int filter_result )
 	}
 }
 
-G::Slot::Signal4<bool,unsigned long,std::string,std::string> & GSmtp::ProtocolMessageStore::doneSignal()
+G::Slot::Signal<bool,unsigned long,const std::string&,const std::string&> & GSmtp::ProtocolMessageStore::doneSignal()
 {
 	return m_done_signal ;
 }
 
-/// \file gprotocolmessagestore.cpp
