@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -18,8 +18,8 @@
 /// \file grequestclient.h
 ///
 
-#ifndef G_REQUEST_CLIENT__H
-#define G_REQUEST_CLIENT__H
+#ifndef G_REQUEST_CLIENT_H
+#define G_REQUEST_CLIENT_H
 
 #include "gdef.h"
 #include "gclient.h"
@@ -33,7 +33,7 @@ namespace GSmtp
 	class RequestClient ;
 }
 
-/// \class GSmtp::RequestClient
+//| \class GSmtp::RequestClient
 /// A network client class that interacts with a remote server using a
 /// stateless line-based request/response protocol.
 ///
@@ -78,18 +78,24 @@ public:
 		///< event signal.
 
 private: // overrides
-	virtual bool onReceive( const char * , size_t , size_t , size_t , char ) override ; // Override from GNet::Client.
-	virtual void onSendComplete() override ; // Override from GNet::BufferedClient.
-	virtual void onDelete( const std::string & ) override ; // Override from GNet::Client.
-	virtual void onSecure( const std::string & , const std::string & ) override ; // Override from GNet::SocketProtocolSink.
-	virtual void onConnect() override ; // Override from GNet::SimpleClient.
+	bool onReceive( const char * , std::size_t , std::size_t , std::size_t , char ) override ; // Override from GNet::Client.
+	void onSendComplete() override ; // Override from GNet::BufferedClient.
+	void onDelete( const std::string & ) override ; // Override from GNet::Client.
+	void onSecure( const std::string & , const std::string & , const std::string & ) override ; // Override from GNet::SocketProtocolSink.
+	void onConnect() override ; // Override from GNet::SimpleClient.
+
+public:
+	~RequestClient() override = default ;
+	RequestClient( const RequestClient & ) = delete ;
+	RequestClient( RequestClient && ) = delete ;
+	void operator=( const RequestClient & ) = delete ;
+	void operator=( RequestClient && ) = delete ;
 
 private:
-	RequestClient( const RequestClient & ) g__eq_delete ;
-	void operator=( const RequestClient & ) g__eq_delete ;
 	void onTimeout() ;
 	std::string requestLine( const std::string & ) const ;
 	std::string result( std::string ) const ;
+	static GNet::Client::Config netConfig( unsigned int connection_timeout , unsigned int response_timeout ) ;
 
 private:
 	std::string m_eol ;
