@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -18,8 +18,8 @@
 /// \file gsmtpclient.h
 ///
 
-#ifndef G_SMTP_CLIENT__H
-#define G_SMTP_CLIENT__H
+#ifndef G_SMTP_CLIENT_H
+#define G_SMTP_CLIENT_H
 
 #include "gdef.h"
 #include "glocation.h"
@@ -45,7 +45,7 @@ namespace GSmtp
 	class ClientProtocol ;
 }
 
-/// \class GSmtp::Client
+//| \class GSmtp::Client
 /// A class which acts as an SMTP client, extracting messages from a
 /// message store and forwarding them to a remote SMTP server.
 ///
@@ -55,20 +55,31 @@ public:
 	struct Config /// A structure containing GSmtp::Client configuration parameters.
 	{
 		std::string filter_address ;
-		unsigned int filter_timeout ;
-		bool bind_local_address ;
+		unsigned int filter_timeout{0U} ;
+		bool bind_local_address{false} ;
 		GNet::Address local_address ;
 		ClientProtocol::Config client_protocol_config ;
-		unsigned int connection_timeout ;
-		unsigned int secure_connection_timeout ;
-		bool secure_tunnel ;
+		unsigned int connection_timeout{0U} ;
+		unsigned int secure_connection_timeout{0U} ;
+		bool secure_tunnel{false} ;
 		std::string sasl_client_config ;
 
-		Config( std::string filter_address , unsigned int filter_timeout ,
+		Config() ;
+		Config( const std::string & filter_address , unsigned int filter_timeout ,
 			bool bind_local_address , const GNet::Address & local_address ,
 			const ClientProtocol::Config & protocol_config ,
 			unsigned int connection_timeout , unsigned int secure_connection_timeout ,
 			bool secure_tunnel , const std::string & sasl_client_config ) ;
+
+		Config & set_filter_address( const std::string & ) ;
+		Config & set_filter_timeout( unsigned int ) ;
+		Config & set_bind_local_address( bool = true ) ;
+		Config & set_local_address( const GNet::Address & ) ;
+		Config & set_client_protocol_config( const ClientProtocol::Config & ) ;
+		Config & set_connection_timeout( unsigned int ) ;
+		Config & set_secure_connection_timeout( unsigned int ) ;
+		Config & set_secure_tunnel( bool = true ) ;
+		Config & set_sasl_client_config( const std::string & ) ;
 	} ;
 
 	Client( GNet::ExceptionSink , const GNet::Location & remote ,
@@ -79,7 +90,7 @@ public:
 			///< repeatedly. Wait for a messageDoneSignal() between
 			///< each sendMessage().
 
-	virtual ~Client() ;
+	~Client() override ;
 		///< Destructor.
 
 	void sendMessagesFrom( MessageStore & store ) ;
@@ -93,7 +104,7 @@ public:
 		///< The messageDoneSignal() is not used when sending
 		///< messages using this method.
 
-	void sendMessage( unique_ptr<StoredMessage> message ) ;
+	void sendMessage( std::unique_ptr<StoredMessage> message ) ;
 		///< Starts sending the given message. Cannot be called
 		///< if there is a message already in the pipeline.
 		///<
@@ -103,24 +114,30 @@ public:
 		///< The message is fail()ed if it cannot be sent. If this
 		///< Client object is deleted before the message is sent
 		///< the message is neither fail()ed or destroy()ed.
+		///<
+		///< Does nothing if there are no message recipients.
 
-	G::Slot::Signal1<std::string> & messageDoneSignal() ;
+	G::Slot::Signal<const std::string&> & messageDoneSignal() ;
 		///< Returns a signal that indicates that sendMessage()
 		///< has completed or failed.
 
 private: // overrides
-	virtual void onConnect() override ; // Override from GNet::SimpleClient.
-	virtual bool onReceive( const char * , size_t , size_t , size_t , char ) override ; // Override from GNet::Client.
-	virtual void onDelete( const std::string & ) override ; // Override from GNet::HeapClient.
-	virtual void onSendComplete() override ; // Override from GNet::BufferedClient.
-	virtual void onSecure( const std::string & , const std::string & ) override ; // Override from GNet::SocketProtocol.
-	virtual bool protocolSend( const std::string & , size_t , bool ) override ; // Override from ClientProtocol::Sender.
+	void onConnect() override ; // Override from GNet::SimpleClient.
+	bool onReceive( const char * , std::size_t , std::size_t , std::size_t , char ) override ; // Override from GNet::Client.
+	void onDelete( const std::string & ) override ; // Override from GNet::HeapClient.
+	void onSendComplete() override ; // Override from GNet::BufferedClient.
+	void onSecure( const std::string & , const std::string & , const std::string & ) override ; // Override from GNet::SocketProtocol.
+	bool protocolSend( const std::string & , std::size_t , bool ) override ; // Override from ClientProtocol::Sender.
+
+public:
+	Client( const Client & ) = delete ;
+	Client( Client && ) = delete ;
+	void operator=( const Client & ) = delete ;
+	void operator=( Client && ) = delete ;
 
 private:
-	Client( const Client & ) g__eq_delete ;
-	void operator=( const Client & ) g__eq_delete ;
-	shared_ptr<StoredMessage> message() ;
-	void protocolDone( int , std::string , std::string ) ; // see ClientProtocol::doneSignal()
+	std::shared_ptr<StoredMessage> message() ;
+	void protocolDone( int , const std::string & , const std::string & , const G::StringArray & ) ; // see ClientProtocol::doneSignal()
 	void filterStart() ;
 	void filterDone( int ) ;
 	bool sendNext() ;
@@ -129,17 +146,28 @@ private:
 	void messageDestroy() ;
 	void startSending() ;
 	void quitAndFinish() ;
+	static GNet::Client::Config netConfig( const Config & smtp_config ) ;
 
 private:
 	MessageStore * m_store ;
 	G::CallStack m_stack ;
-	unique_ptr<Filter> m_filter ;
-	shared_ptr<StoredMessage> m_message ;
-	MessageStore::Iterator m_iter ;
+	std::unique_ptr<Filter> m_filter ;
+	std::shared_ptr<StoredMessage> m_message ;
+	std::shared_ptr<MessageStore::Iterator> m_iter ;
 	ClientProtocol m_protocol ;
 	bool m_secure_tunnel ;
-	G::Slot::Signal1<std::string> m_message_done_signal ;
+	G::Slot::Signal<const std::string&> m_message_done_signal ;
 	unsigned int m_message_count ;
 } ;
 
+inline GSmtp::Client::Config & GSmtp::Client::Config::set_filter_address( const std::string & s ) { filter_address = s ; return *this ; }
+inline GSmtp::Client::Config & GSmtp::Client::Config::set_filter_timeout( unsigned int t ) { filter_timeout = t ; return *this ; }
+inline GSmtp::Client::Config & GSmtp::Client::Config::set_bind_local_address( bool b ) { bind_local_address = b ; return *this ; }
+inline GSmtp::Client::Config & GSmtp::Client::Config::set_local_address( const GNet::Address & a ) { local_address = a ; return *this ; }
+inline GSmtp::Client::Config & GSmtp::Client::Config::set_client_protocol_config( const ClientProtocol::Config & c ) { client_protocol_config = c ; return *this ; }
+inline GSmtp::Client::Config & GSmtp::Client::Config::set_connection_timeout( unsigned int t ) { connection_timeout = t ; return *this ; }
+inline GSmtp::Client::Config & GSmtp::Client::Config::set_secure_connection_timeout( unsigned int t ) { secure_connection_timeout = t ; return *this ; }
+inline GSmtp::Client::Config & GSmtp::Client::Config::set_secure_tunnel( bool b ) { secure_tunnel = b ; return *this ; }
+inline GSmtp::Client::Config & GSmtp::Client::Config::set_sasl_client_config( const std::string & s ) { sasl_client_config = s ; return *this ; }
+
 #endif
