@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -18,8 +18,8 @@
 /// \file gspamclient.h
 ///
 
-#ifndef G_SMTP_SPAM_CLIENT__H
-#define G_SMTP_SPAM_CLIENT__H
+#ifndef G_SMTP_SPAM_CLIENT_H
+#define G_SMTP_SPAM_CLIENT_H
 
 #include "gdef.h"
 #include "gclient.h"
@@ -36,7 +36,7 @@ namespace GSmtp
 	class SpamClient ;
 }
 
-/// \class GSmtp::SpamClient
+//| \class GSmtp::SpamClient
 /// A client class that interacts with a remote process using a
 /// protocol somewhat similar to the spamassassin spamc/spamd
 /// protocol. The interface is similar to GSmtp::RequestClient
@@ -72,17 +72,23 @@ public:
 		///< Sets the username used in the network protocol.
 
 private: // overrides
-	virtual void onConnect() override ; // Override from GNet::SimpleClient.
-	virtual bool onReceive( const char * , size_t , size_t , size_t , char ) override ; // Override from GNet::Client.
-	virtual void onSendComplete() override ; // Override from GNet::SimpleClient.
-	virtual void onSecure( const std::string & , const std::string & ) override ; // Override from GNet::SocketProtocolSink.
-	virtual void onDelete( const std::string & ) override ; // Override from GNet::HeapClient.
+	void onConnect() override ; // Override from GNet::SimpleClient.
+	bool onReceive( const char * , std::size_t , std::size_t , std::size_t , char ) override ; // Override from GNet::Client.
+	void onSendComplete() override ; // Override from GNet::SimpleClient.
+	void onSecure( const std::string & , const std::string & , const std::string & ) override ; // Override from GNet::SocketProtocolSink.
+	void onDelete( const std::string & ) override ; // Override from GNet::HeapClient.
+
+public:
+	~SpamClient() override = default ;
+	SpamClient( const SpamClient & ) = delete ;
+	SpamClient( SpamClient && ) = delete ;
+	void operator=( const SpamClient & ) = delete ;
+	void operator=( SpamClient && ) = delete ;
 
 private:
-	SpamClient( const SpamClient & ) g__eq_delete ;
-	void operator=( const SpamClient & ) g__eq_delete ;
 	void onTimeout() ;
 	void start() ;
+	static GNet::Client::Config netConfig( unsigned int connect_timeout , unsigned int response_timeout ) ;
 
 private:
 	struct Request
@@ -99,6 +105,10 @@ private:
 	{
 		explicit Response( bool read_only ) ;
 		~Response() ;
+		Response( const Response & ) = delete ;
+		Response( Response && ) = delete ;
+		void operator=( const Response & ) = delete ;
+		void operator=( Response && ) = delete ;
 		void add( const std::string & , const std::string & ) ;
 		bool ok( const std::string & ) const ;
 		bool complete() const ;
@@ -108,8 +118,8 @@ private:
 		std::string m_path_tmp ;
 		std::string m_path_final ;
 		std::ofstream m_stream ;
-		size_t m_content_length ;
-		size_t m_size ;
+		std::size_t m_content_length ;
+		std::size_t m_size ;
 		std::string m_result ;
 	} ;
 
