@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -18,8 +18,8 @@
 /// \file gverifier.h
 ///
 
-#ifndef G_SMTP_VERIFIER__H
-#define G_SMTP_VERIFIER__H
+#ifndef G_SMTP_VERIFIER_H
+#define G_SMTP_VERIFIER_H
 
 #include "gdef.h"
 #include "gverifierstatus.h"
@@ -33,7 +33,7 @@ namespace GSmtp
 	class Verifier ;
 }
 
-/// \class GSmtp::Verifier
+//| \class GSmtp::Verifier
 /// An asynchronous interface that verifies recipient 'to' addresses.
 /// This is used in the VRFY and RCPT commands in the smtp server
 /// protocol.
@@ -52,7 +52,7 @@ public:
 			///< The 'mail-from' address is passed in for RCPT commands, but
 			///< not VRFY.
 
-	virtual G::Slot::Signal2<std::string,VerifierStatus> & doneSignal() = 0 ;
+	virtual G::Slot::Signal<const std::string&,const VerifierStatus&> & doneSignal() = 0 ;
 		///< Returns a signal that is emit()ed when the verify() request
 		///< is complete. The first signal parameter is the mailbox
 		///< name (ie. rcpt_to_parameter).
@@ -60,7 +60,7 @@ public:
 	virtual void cancel() = 0 ;
 		///< Aborts any current processing.
 
-	virtual ~Verifier() ;
+	virtual ~Verifier() = default ;
 		///< Destructor.
 } ;
 
