@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -18,8 +18,8 @@
 /// \file gverifierstatus.h
 ///
 
-#ifndef G_SMTP_VERIFIER_STATUS__H
-#define G_SMTP_VERIFIER_STATUS__H
+#ifndef G_SMTP_VERIFIER_STATUS_H
+#define G_SMTP_VERIFIER_STATUS_H
 
 #include "gdef.h"
 #include <string>
@@ -29,7 +29,7 @@ namespace GSmtp
 	class VerifierStatus ;
 }
 
-/// \class GSmtp::VerifierStatus
+//| \class GSmtp::VerifierStatus
 /// A structure returned by GSmtp::Verifier to describe the status of
 /// a 'rcpt-to' recipient.
 ///
@@ -50,15 +50,6 @@ namespace GSmtp
 class GSmtp::VerifierStatus
 {
 public:
-	bool is_valid ;
-	bool is_local ;
-	bool temporary ;
-	bool abort ;
-	std::string full_name ;
-	std::string address ;
-	std::string response ;
-	std::string reason ;
-
 	VerifierStatus() ;
 		///< Default constructor for an invalid remote mailbox.
 
@@ -73,6 +64,17 @@ public:
 	std::string str( const std::string & to ) const ;
 		///< Returns a string representation of the structure
 		///< plus the original recipient 'to' address.
+
+public:
+	bool is_valid{false} ;
+	bool is_local{false} ;
+	bool temporary{false} ;
+	bool abort{false} ;
+	std::string full_name ;
+	std::string address ;
+	std::string response ;
+	std::string reason ;
+
 } ;
 
 #endif
