@@ -1,66 +1,54 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
-//
-// gdialog.cpp
-//
+///
+/// \file gdialog.cpp
+///
 
 #include "gdef.h"
-#include "qt.h"
+#include "gqt.h"
 #include "gdialog.h"
 #include "gfile.h"
 #include "glog.h"
 #include <algorithm>
 #include <stdexcept>
 
-GDialog::GDialog( bool with_help , const G::Path & virgin_flag_file ) :
+GDialog::GDialog( const G::Path & virgin_flag_file , bool with_launch ) :
 	QDialog(nullptr) ,
 	m_first(true) ,
-	m_help_button(nullptr) ,
+	m_with_launch(with_launch) ,
+	m_next_is_launch(false) ,
 	m_cancel_button(nullptr) ,
 	m_back_button(nullptr) ,
 	m_next_button(nullptr) ,
 	m_finish_button(nullptr) ,
-	m_waiting(false) ,
+	m_finishing(false) ,
 	m_virgin_flag_file(virgin_flag_file)
 {
-	init( with_help ) ;
-}
-
-void GDialog::init( bool with_help )
-{
-	if( with_help )
-	{
-		m_help_button = new QPushButton(tr("&Help")) ;
-		connect( m_help_button, SIGNAL(clicked()) , this , SLOT(helpButtonClicked()) ) ;
-	}
-
 	m_cancel_button = new QPushButton(tr("Cancel")) ;
-	m_back_button = new QPushButton(tr("< &Back")) ;
-	m_next_button = new QPushButton(tr("Next >")) ;
+	m_back_button = new QPushButton(tr("&< Back")) ;
+	m_next_button = new QPushButton(tr("Next &>")) ;
 	m_finish_button = new QPushButton(tr("&Finish")) ;
 
-	connect( m_back_button, SIGNAL(clicked()), this, SLOT(backButtonClicked()) ) ;
-	connect( m_next_button, SIGNAL(clicked()), this, SLOT(nextButtonClicked()) ) ;
-	connect( m_cancel_button, SIGNAL(clicked()), this, SLOT(reject()) ) ;
-	connect( m_finish_button, SIGNAL(clicked()) , this , SLOT(finishButtonClicked()) ) ;
+	connect( m_back_button , SIGNAL(clicked()) , this, SLOT(backButtonClicked()) ) ;
+	connect( m_next_button , SIGNAL(clicked()) , this, SLOT(nextButtonClicked()) ) ;
+	connect( m_cancel_button , SIGNAL(clicked()) , this, SLOT(reject()) ) ;
+	connect( m_finish_button , SIGNAL(clicked()) , this , SLOT(finishButtonClicked()) ) ;
 
 	m_button_layout = new QHBoxLayout ;
-	if( m_help_button != nullptr )
-		m_button_layout->addWidget( m_help_button ) ;
 	m_button_layout->addStretch( 1 ) ;
 	m_button_layout->addWidget( m_cancel_button ) ;
 	m_button_layout->addWidget( m_back_button ) ;
@@ -106,7 +94,7 @@ GPage & GDialog::page( const std::string & name )
 
 const GPage & GDialog::page( const std::string & name ) const
 {
-	Map::const_iterator p = m_map.find(name) ;
+	auto p = m_map.find( name ) ;
 	if( p == m_map.end() )
 		throw std::runtime_error( std::string() + "internal error: no such page: " + name ) ;
 	return *(*p).second ;
@@ -118,13 +106,6 @@ void GDialog::setFirstPage( GPage & page )
 	switchPage( m_history.back() ) ;
 }
 
-void GDialog::helpButtonClicked()
-{
-	std::string base = "http://emailrelay.sourceforge.net/help/" ;
-	std::string url = base + page(currentPageName()).helpName() + ".html" ;
-	QDesktopServices::openUrl( QString(url.c_str()) ) ;
-}
-
 void GDialog::backButtonClicked()
 {
 	std::string old_page_name = m_history.back();
@@ -134,23 +115,32 @@ void GDialog::backButtonClicked()
 
 void GDialog::nextButtonClicked()
 {
-	std::string old_page_name = m_history.back();
-	std::string new_page_name = page(old_page_name).nextPage();
-	m_history.push_back(new_page_name);
-	switchPage( m_history.back() , old_page_name ) ;
+	if( m_next_is_launch )
+	{
+		m_next_button->setEnabled( false ) ;
+		page(m_history.back()).onLaunch() ;
+	}
+	else
+	{
+		std::string old_page_name = m_history.back() ;
+		std::string new_page_name = page(old_page_name).nextPage() ;
+		m_history.push_back( new_page_name ) ;
+		switchPage( m_history.back() , old_page_name ) ;
+	}
 }
 
 void GDialog::finishButtonClicked()
 {
-	std::string old_page_name = m_history.back();
-	std::string new_page_name = page(old_page_name).nextPage();
-	if( new_page_name.empty() )
+	if( page(m_history.back()).isFinishPage() ) // ie. "close" button clicked
 	{
-		accept() ;
+		accept() ; // QDialog::accept() terminates the modal dialog box
 	}
 	else
 	{
-		m_history.push_back(new_page_name);
+		// next -- ie. ready-to-finish -> finish page
+		std::string old_page_name = m_history.back() ;
+		std::string new_page_name = page(old_page_name).nextPage() ;
+		m_history.push_back( new_page_name ) ;
 		switchPage( m_history.back() , old_page_name ) ;
 	}
 }
@@ -159,45 +149,51 @@ void GDialog::pageUpdated()
 {
 	std::string current_page_name = m_history.back() ;
 	G_DEBUG( "GDialog::pageUpdated: \"" << current_page_name << "\" page updated" ) ;
+	GPage & current_page = page( current_page_name ) ;
 
-	GPage & current_page = page(current_page_name) ;
-	if( m_waiting )
+	if( current_page.isReadyToFinishPage() )
 	{
-		; // no-op
+		// ready to finish -- no next button
+		m_cancel_button->setEnabled( true ) ;
+		m_back_button->setEnabled( m_history.size() != 1U ) ;
+		m_next_button->setEnabled( false ) ;
+		m_finish_button->setEnabled( true ) ;
 	}
-	else if( current_page.closeButton() )
+	else if( current_page.isFinishPage() && current_page.isFinishing() )
 	{
-		m_cancel_button->setEnabled(false) ;
-		m_back_button->setEnabled(false) ;
-		m_next_button->setEnabled(false) ;
-		m_finish_button->setText(tr("Close")) ;
-		m_finish_button->setEnabled(true) ;
-		if( m_help_button != nullptr )
-			m_help_button->setEnabled(!current_page.helpName().empty()) ;
-		if( m_virgin_flag_file != G::Path() )
-			G::File::remove( m_virgin_flag_file , G::File::NoThrow() ) ;
+		// finishing -- all disabled
+		m_cancel_button->setEnabled( false ) ; // moot
+		m_back_button->setEnabled( false ) ;
+		m_next_button->setEnabled( false ) ;
+		m_finish_button->setEnabled( false ) ;
+	}
+	else if( current_page.isFinishPage() && !current_page.isComplete() )
+	{
+		// finishing failed -- can go back
+		m_cancel_button->setEnabled( true ) ;
+		m_back_button->setEnabled( m_history.size() != 1U ) ;
+		m_next_button->setEnabled( false ) ;
+		m_finish_button->setEnabled( false ) ;
+	}
+	else if( current_page.isFinishPage() && current_page.isComplete() )
+	{
+		// finished ok -- close or launch
+		m_cancel_button->setEnabled( false ) ;
+		m_back_button->setEnabled( false ) ;
+		m_next_button->setEnabled( m_next_is_launch = m_with_launch && current_page.canLaunch() ) ;
+		m_finish_button->setText( tr("Close") ) ;
+		m_finish_button->setEnabled( true ) ;
+		if( !m_virgin_flag_file.empty() )
+			G::File::remove( m_virgin_flag_file , std::nothrow ) ;
+		if( m_next_is_launch )
+			m_next_button->setText( tr("Launch") ) ;
 	}
 	else
 	{
-		m_cancel_button->setEnabled(true) ;
+		m_cancel_button->setEnabled( true ) ;
 		m_back_button->setEnabled( m_history.size() != 1U ) ;
-
-		// enable either 'next' or 'finish'...
-		m_finish_button->setText(tr("&Finish")) ;
-		bool finish_button = current_page.useFinishButton() ;
-		QPushButton * active_button = finish_button ? m_finish_button : m_next_button ;
-		QPushButton * inactive_button = finish_button ? m_next_button : m_finish_button ;
-
-		// ...or neither
-		bool active_state = current_page.isComplete() ;
-		if( active_button == m_next_button && current_page.nextPage().empty() )
-			active_state = false ;
-
-		active_button->setEnabled( active_state ) ;
-		inactive_button->setEnabled( false ) ;
-
-		if( m_help_button != nullptr )
-			m_help_button->setEnabled(!current_page.helpName().empty()) ;
+		m_next_button->setEnabled( current_page.isComplete() ) ;
+		m_finish_button->setEnabled( false ) ;
 	}
 }
 
@@ -210,17 +206,17 @@ void GDialog::switchPage( const std::string & new_page_name , const std::string
 		GPage & oldPage = page(old_page_name) ;
 		oldPage.hide();
 		m_main_layout->removeWidget(&oldPage);
-		disconnect(&oldPage, SIGNAL(pageUpdateSignal()), this, SLOT(pageUpdated()));
+		disconnect( &oldPage , SIGNAL(pageUpdateSignal()) , this , SLOT(pageUpdated()) );
 	}
 
 	// show and connect the new page
 	//
-	GPage & new_page = page(new_page_name) ;
-	m_main_layout->insertWidget(0, &new_page);
+	GPage & new_page = page( new_page_name ) ;
+	m_main_layout->insertWidget( 0 , &new_page ) ;
 	new_page.onShow( back ) ; // GPage method
 	new_page.show() ;
 	new_page.setFocus() ;
-	connect(&new_page, SIGNAL(pageUpdateSignal()), this, SLOT(pageUpdated())) ;
+	connect( &new_page , SIGNAL(pageUpdateSignal()) , this , SLOT(pageUpdated()) ) ;
 
 	// modify the next and finish buttons according to the page state
 	//
@@ -240,7 +236,7 @@ std::string GDialog::currentPageName() const
 GPage & GDialog::previousPage( unsigned int distance )
 {
 	if( m_history.size() < (distance+1U) ) throw std::runtime_error("internal error") ;
-	History::reverse_iterator p = m_history.rbegin() ;
+	auto p = m_history.rbegin() ;
 	while( distance-- ) ++p ;
 	G_DEBUG( "GDialog::previousPage: " << currentPageName() << " -> " << *p ) ;
 	return page(*p) ;
@@ -258,31 +254,7 @@ void GDialog::dumpInstallVariables( std::ostream & stream ) const
 
 void GDialog::dump( std::ostream & stream , bool for_install ) const
 {
-	for( History::const_iterator p = m_history.begin() ; p != m_history.end() ; ++p )
-		page(*p).dump( stream , for_install ) ;
+	for( const auto & pname : m_history )
+		page(pname).dump( stream , for_install ) ;
 }
 
-void GDialog::wait( bool wait_on )
-{
-	if( wait_on && !m_waiting )
-	{
-		m_waiting = wait_on ;
-
-		m_back_state = m_back_button->isEnabled() ;
-		m_next_state = m_next_button->isEnabled() ;
-		m_finish_state = m_finish_button->isEnabled() ;
-
-		// all off
-		m_cancel_button->setEnabled(false);
-		m_back_button->setEnabled(false);
-		m_next_button->setEnabled(false);
-		m_finish_button->setEnabled(false);
-	}
-	else if( m_waiting && !wait_on )
-	{
-		m_waiting = wait_on ;
-		pageUpdated() ;
-	}
-}
-
-/// \file gdialog.cpp
