@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -18,11 +18,11 @@
 /// \file gdialog.h
 ///
 
-#ifndef G_DIALOG_H
-#define G_DIALOG_H
+#ifndef G_MAIN_GUI_DIALOG_H
+#define G_MAIN_GUI_DIALOG_H
 
 #include "gdef.h"
-#include "qt.h"
+#include "gqt.h"
 #include "gpage.h"
 #include "gpath.h"
 #include <list>
@@ -32,13 +32,13 @@ class QPushButton;
 class QVBoxLayout;
 class GPage ;
 
-/// \class GDialog
+//| \class GDialog
 /// The main forward-back dialog box.
 ///
 class GDialog : public QDialog
 {Q_OBJECT
 public:
-	GDialog( bool with_help , const G::Path & virgin_flag_file ) ;
+	GDialog( const G::Path & virgin_flag_file , bool with_launch ) ;
 		///< Constructor. Use a sequence of add()s to initialise
 		///< ending with add(void).
 
@@ -46,8 +46,8 @@ public:
 		///< Adds a page.
 
 	void add( GPage * , const std::string & conditional_page_name ) ;
-		///< Adds a page but only if the the page name
-		///< matches the given conditional name or if the given
+		///< Adds a page but only if the the page name matches
+		///< the given conditional name or if the given
 		///< conditional name is empty.
 
 	void add() ;
@@ -77,28 +77,29 @@ public:
 	void dumpInstallVariables( std::ostream & ) const ;
 		///< Dump the install variables from all the pages.
 
-	void wait( bool ) ;
-		///< Disables all buttons.
+	void setFinishing( bool ) ;
+		///< Disables all buttons. See also GPage::allFinished()
+		///< and GPage::nearlyAllFinished().
 
 private slots:
-	void helpButtonClicked() ;
 	void backButtonClicked() ;
 	void nextButtonClicked() ;
 	void finishButtonClicked() ;
 	void pageUpdated() ;
 
 private:
-	void init( bool ) ;
 	void dump( std::ostream & , bool for_install ) const ;
 	void setFirstPage( GPage & ) ;
 	void switchPage( const std::string & new_page_name , const std::string & old_page_name = std::string() , bool back = false ) ;
 
 private:
-	typedef std::list<std::string> History ;
-	typedef std::map<std::string,GPage*> Map ;
+	using History = std::list<std::string> ;
+	using Map = std::map<std::string,GPage*> ;
 	Map m_map ;
 	History m_history ;
 	bool m_first ;
+	bool m_with_launch ;
+	bool m_next_is_launch ;
 	QPushButton * m_help_button ;
 	QPushButton * m_cancel_button ;
 	QPushButton * m_back_button ;
@@ -106,7 +107,7 @@ private:
 	QPushButton * m_finish_button ;
 	QHBoxLayout * m_button_layout ;
 	QVBoxLayout * m_main_layout ;
-	bool m_waiting ;
+	bool m_finishing ;
 	bool m_back_state ;
 	bool m_next_state ;
 	bool m_finish_state ;
