@@ -1,22 +1,22 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
-//
-// glink_unix.cpp
-//
+///
+/// \file glink_unix.cpp
+///
 
 #include "gdef.h"
 #include "gstr.h"
@@ -26,37 +26,42 @@
 #include <sstream>
 #include <fstream>
 
-class GLinkImp
+class G::LinkImp
 {
 public:
-	GLinkImp( const G::Path & target_path , const std::string & name , const std::string & description ,
-		const G::Path & working_dir , const G::StringArray & args , const G::Path & icon_source , GLink::Show show ,
+	LinkImp( const Path & target_path , const std::string & name , const std::string & description ,
+		const Path & working_dir , const StringArray & args , const Path & icon_source , Link::Show show ,
 		const std::string & c1 , const std::string & c2 , const std::string & c3 ) ;
 	static std::string filename( const std::string & ) ;
-	void saveAs( const G::Path & ) ;
+	void saveAs( const Path & ) ;
+
+public:
+	~LinkImp() = default ;
+	LinkImp( const LinkImp & ) = delete ;
+	LinkImp( LinkImp && ) = delete ;
+	void operator=( const LinkImp & ) = delete ;
+	void operator=( LinkImp && ) = delete ;
 
 private:
-	GLinkImp( const GLinkImp & ) ;
-	void operator=( const GLinkImp & ) ;
 	static std::string quote( const std::string & ) ;
 	static std::string escape( const std::string & ) ;
-	static std::string escapeAndQuote( const G::StringArray & ) ;
+	static std::string escapeAndQuote( const StringArray & ) ;
 
 private:
-	G::Path m_target_path ;
+	Path m_target_path ;
 	std::string m_name ;
 	std::string m_description ;
-	G::Path m_working_dir ;
-	G::StringArray m_args ;
-	G::Path m_icon_source ;
+	Path m_working_dir ;
+	StringArray m_args ;
+	Path m_icon_source ;
 	bool m_terminal ;
 	std::string m_c1 ;
 	std::string m_c2 ;
 	std::string m_c3 ;
 } ;
 
-GLinkImp::GLinkImp( const G::Path & target_path , const std::string & name , const std::string & description ,
-	const G::Path & working_dir , const G::StringArray & args , const G::Path & icon_source , GLink::Show show ,
+G::LinkImp::LinkImp( const Path & target_path , const std::string & name , const std::string & description ,
+	const Path & working_dir , const StringArray & args , const Path & icon_source , Link::Show show ,
 	const std::string & c1 , const std::string & c2 , const std::string & c3 ) :
 		m_target_path(target_path) ,
 		m_name(name) ,
@@ -64,28 +69,29 @@ GLinkImp::GLinkImp( const G::Path & target_path , const std::string & name , con
 		m_working_dir(working_dir) ,
 		m_args(args) ,
 		m_icon_source(icon_source) ,
-		m_terminal(show==GLink::Show::Default) ,
+		m_terminal(show==Link::Show::Default) ,
 		m_c1(c1) ,
 		m_c2(c2) ,
 		m_c3(c3)
 {
 }
 
-std::string GLinkImp::filename( const std::string & name )
+std::string G::LinkImp::filename( const std::string & name )
 {
-	std::string result = G::Str::lower(name) + ".desktop" ;
-	G::Str::replaceAll( result , "-" , "" ) ;
+	std::string result = Str::lower(name) + ".desktop" ;
+	Str::replaceAll( result , "-" , "" ) ;
 	return result ;
 }
 
-void GLinkImp::saveAs( const G::Path & path )
+void G::LinkImp::saveAs( const Path & path )
 {
 	// see "http://standards.freedesktop.org"
 
 	// TODO maybe use "xdg-desktop-menu"/"xdg-desktop-icon"
 
 	const char * eol = "\n" ;
-	std::ofstream file( path.str().c_str() ) ;
+	std::ofstream file ;
+	File::open( file , path ) ;
 
 	if( !m_c1.empty() ) file << "# " << m_c1 << eol ;
 	if( !m_c2.empty() ) file << "# " << m_c2 << eol ;
@@ -109,12 +115,12 @@ void GLinkImp::saveAs( const G::Path & path )
 
 	file.flush() ;
 	if( !file.good() )
-		throw GLink::SaveError(path.str()) ;
+		throw Link::SaveError(path.str()) ;
 
-	G::File::chmodx( path , G::File::NoThrow() ) ;
+	File::chmodx( path , std::nothrow ) ;
 }
 
-std::string GLinkImp::escape( const std::string & s_in )
+std::string G::LinkImp::escape( const std::string & s_in )
 {
 	// <citation version="1.0">
 	// Note that the general escape rule for values of type string states that
@@ -130,10 +136,10 @@ std::string GLinkImp::escape( const std::string & s_in )
 	// percentage characters must be escaped as %%.
 	// </citation>
 
-	return G::Str::escaped( G::Str::escaped(s_in,'\\',"\\$","\\$") , '%' , "%" , "%" ) ;
+	return Str::escaped( Str::escaped(s_in,'\\',"\\$","\\$") , '%' , "%" , "%" ) ;
 }
 
-std::string GLinkImp::quote( const std::string & s_in )
+std::string G::LinkImp::quote( const std::string & s_in )
 {
 	// <citation version="1.0">
 	// If an argument contains a reserved character the argument must be quoted.
@@ -153,7 +159,7 @@ std::string GLinkImp::quote( const std::string & s_in )
 		// character ("\") by preceding it with an additional backslash character.
 		// </citation>
 
-		return std::string(1U,'\"') + G::Str::escaped(s_in,'\\',"\"`$\\","\"`$\\") + std::string(1U,'\"') ;
+		return std::string(1U,'\"') + Str::escaped(s_in,'\\',"\"`$\\","\"`$\\") + std::string(1U,'\"') ;
 	}
 	else
 	{
@@ -161,13 +167,13 @@ std::string GLinkImp::quote( const std::string & s_in )
 	}
 }
 
-std::string GLinkImp::escapeAndQuote( const G::StringArray & args )
+std::string G::LinkImp::escapeAndQuote( const StringArray & args )
 {
 	std::ostringstream ss ;
 	const char * sep = "" ;
-	for( G::StringArray::const_iterator p = args.begin() ; p != args.end() ; ++p )
+	for( const auto & arg : args )
 	{
-		ss << sep << quote(escape(*p)) ;
+		ss << sep << quote(escape(arg)) ;
 		sep = " " ;
 	}
 	return ss.str() ;
@@ -175,36 +181,33 @@ std::string GLinkImp::escapeAndQuote( const G::StringArray & args )
 
 // ==
 
-GLink::GLink( const G::Path & target_path , const std::string & name , const std::string & description ,
-	const G::Path & working_dir , const G::StringArray & args , const G::Path & icon_source , Show show ,
+G::Link::Link( const Path & target_path , const std::string & name , const std::string & description ,
+	const Path & working_dir , const StringArray & args , const Path & icon_source , Show show ,
 	const std::string & c1 , const std::string & c2 , const std::string & c3 ) :
-		m_imp( new GLinkImp(target_path,name,description,working_dir,args,icon_source,show,c1,c2,c3) )
+		m_imp(std::make_unique<LinkImp>(target_path,name,description,working_dir,args,icon_source,show,c1,c2,c3))
 {
 }
 
-std::string GLink::filename( const std::string & name_in )
+G::Link::~Link()
+= default ;
+
+std::string G::Link::filename( const std::string & name_in )
 {
-	return GLinkImp::filename( name_in ) ;
+	return LinkImp::filename( name_in ) ;
 }
 
-void GLink::saveAs( const G::Path & link_path )
+void G::Link::saveAs( const Path & link_path )
 {
 	m_imp->saveAs( link_path ) ;
 }
 
-GLink::~GLink()
+bool G::Link::exists( const Path & path )
 {
-	delete m_imp ;
+	return File::isLink( path , std::nothrow ) ;
 }
 
-bool GLink::exists( const G::Path & path )
+bool G::Link::remove( const Path & link_path )
 {
-	return G::File::isLink( path ) ;
+	return File::remove( link_path , std::nothrow ) ;
 }
 
-bool GLink::remove( const G::Path & link_path )
-{
-	return G::File::remove( link_path , G::File::NoThrow() ) ;
-}
-
-/// \file glink_unix.cpp
