@@ -1,22 +1,22 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
-//
-// guimain.cpp
-//
+///
+/// \file guimain.cpp
+///
 // Modes
 // -----
 // This GUI program is primarily intended to do a new installation, but it
@@ -50,9 +50,9 @@
 // If there is no payload and no pointer file then all bets are off.
 //
 // The name of the pointer file is the name of the GUI executable without any
-// extension; or if the GUI executable had no extension to begin with then it
-// is the name of the GUI executable with ".cfg" appended; or on a mac it is
-// just "dir.cfg" somewhere inside the bundle.
+// extension; or if the GUI executable had no extension to begin with then
+// it is the name of the GUI executable with ".cfg" appended; or on a mac
+// it is just "dir.cfg" somewhere inside the bundle.
 //
 // The main pointer variables are "dir-config", "dir-install" and "dir-run".
 // On unix these might be "/etc", "/usr/bin", and "/run", respectively.
@@ -96,7 +96,8 @@
 // is a startup batch file on windows), and the authentication secrets file.
 //
 // On unix the server configuration file is in a format that can be read by the
-// System V init script to assemble the server's full command-line.
+// System V init script to assemble the server's full command-line (although
+// it is no longer done that way).
 //
 // The install directory, and to a lesser extent the configuration directory,
 // will contain additional files copied straight from the payload, such as
@@ -104,7 +105,7 @@
 //
 
 #include "gdef.h"
-#include "qt.h"
+#include "gqt.h"
 #include "gdialog.h"
 #include "gfile.h"
 #include "gbatchfile.h"
@@ -122,18 +123,17 @@
 #include "garg.h"
 #include "gpath.h"
 #include "gstr.h"
+#include "gstringwrap.h"
 #include <string>
 #include <iostream>
 #include <stdexcept>
 #include <fstream>
 
 #ifdef G_WINDOWS
-#if defined(QT_VERSION) && QT_VERSION >= 0x050000
 #ifdef G_QT_STATIC
 Q_IMPORT_PLUGIN(QWindowsIntegrationPlugin)
 #endif
 #endif
-#endif
 
 static int width()
 {
@@ -152,7 +152,7 @@ static int height()
 
 static void error( const std::string & what )
 {
-	QString title(QMessageBox::tr("E-MailRelay")) ;
+	QString title( GQt::qstr("E-MailRelay") ) ;
 	QMessageBox::critical( nullptr , title ,
 		QMessageBox::tr("Failed with the following exception: %1").arg(what.c_str()) ,
 		QMessageBox::Abort , QMessageBox::NoButton , QMessageBox::NoButton ) ;
@@ -160,7 +160,7 @@ static void error( const std::string & what )
 
 static void info( const std::string & text )
 {
-	QString title(QMessageBox::tr("E-MailRelay")) ;
+	QString title( GQt::qstr("E-MailRelay") ) ;
 	QMessageBox::information( nullptr , title , QString::fromLatin1(text.c_str()) ,
 		QMessageBox::Ok , QMessageBox::NoButton , QMessageBox::NoButton ) ;
 }
@@ -187,7 +187,7 @@ class Application : public QApplication
 {
 public:
 	Application( int & argc , char * argv [] ) ;
-	virtual bool notify( QObject * p1 , QEvent * p2 ) ;
+	bool notify( QObject * p1 , QEvent * p2 ) override ;
 } ;
 Application::Application( int & argc , char * argv [] ) :
 	QApplication(argc,argv)
@@ -203,7 +203,7 @@ bool Application::notify( QObject * p1 , QEvent * p2 )
 	{
 		G_ERROR( "exception: " << e.what() ) ;
 		error( e.what() ) ;
-		std::string message = G::Str::wrap( e.what() , "" , "" , 40 ) ;
+		std::string message = G::StringWrap::wrap( e.what() , "" , "" , 40U ) ;
 		qCritical( "exception: %s" , message.c_str() ) ;
 		exit( 3 ) ;
 	}
@@ -221,8 +221,8 @@ static G::Path search( const G::Path & base , const std::string & filename ,
 
 static std::string pointerFilename( const G::Path & argv0 )
 {
-	std::string ext = argv0.basename().find('.') == std::string::npos || isWindows() ? ".cfg" : "" ;
 	std::string filename = argv0.withoutExtension().basename() ;
+	std::string ext = argv0.basename().find('.') == std::string::npos || isWindows() ? ".cfg" : "" ;
 	filename.append( ext ) ;
 	return filename ;
 }
@@ -234,6 +234,18 @@ static G::Path configFile( const G::Path & dir_config )
 		(dir_config+"emailrelay.conf") ;
 }
 
+static std::string value( const G::Arg & args , const std::string & option )
+{
+	std::size_t pos = args.index( option , 1U ) ;
+	std::size_t pos_eq = args.match( option+"=" ) ;
+	if( pos )
+		return args.v( pos + 1U ) ;
+	else if( pos_eq )
+		return G::Str::tail( args.v(pos_eq) , "=" ) ;
+	else
+		return std::string() ;
+}
+
 int main( int argc , char * argv [] )
 {
 	try
@@ -249,17 +261,15 @@ int main( int argc , char * argv [] )
 			info( ss.str() ) ;
 			return 1 ;
 		}
-		G::LogOutput log_ouptut(
-			"emailrelay-gui" ,
-			true , // output
-			true , // with-logging
-			args.contains("-v") , // with-verbose-logging
-			args.contains("-v") , // with-debug
-			true , // with-level
-			false , // with-timestamp
-			false , // strip-context
-			args.contains("-v") // use-syslog
-		) ;
+		G::LogOutput log_ouptut( "" ,
+			G::LogOutput::Config()
+				.set_output_enabled(true)
+				.set_summary_info(true)
+				.set_verbose_info(args.contains("-v"))
+				.set_debug(args.count("-v")>2U)
+				.set_strip(args.count("-v")<2U)
+				.set_with_level(true)
+				.set_use_syslog(args.contains("-v")) ) ;
 		G_LOG( "main: start: " << argv[0] ) ;
 
 		try
@@ -267,16 +277,45 @@ int main( int argc , char * argv [] )
 			G::Path argv0 = G::Arg::exe().empty() ? args.v(0U) : G::Arg::exe() ;
 			bool is_mac = isMac() || args.contains("--as-mac") ;
 
-			// test-mode -- create a minimal payload
+			// load translations from 'translations/emailrelay*.qm' files according
+			// to the 'LANG' environment variable or an explicit '--qm' option
+			QTranslator translator ;
+			{
+				G_LOG( "main: locale: " << GQt::stdstr(QLocale::system().name()) ) ; // eg. "en_GB"
+				bool loaded = false ;
+				G::Path qmfile = value( args , "--qm" ) ;
+				if( !qmfile.empty() )
+					loaded = translator.load( GQt::qstr(qmfile.str()) ) ;
+				if( !loaded )
+					loaded = translator.load( QLocale() , GQt::qstr("emailrelay") , GQt::qstr(".") ,
+						GQt::qstr((argv0.dirname()+"translations").str()) ) ;
+				if( loaded )
+					QCoreApplication::installTranslator( &translator ) ;
+				else
+					G_LOG( "main: no translations loaded" ) ;
+			}
+
+			// test-mode -- create a minimal payload, make it easier to
+			// click through and write install variables to installer.txt
 			if( args.contains("--test") )
 			{
 				G::Path pdir = argv0.dirname() + "payload" ;
-				G::File::mkdir( pdir , G::File::NoThrow() ) ;
-				G::File::copy( argv0 , pdir + argv0.basename() , G::File::NoThrow() ) ;
-				std::ofstream f( (pdir+"payload.cfg").str().c_str() ) ;
-				std::string sdir = isWindows() ? "" : "sbin/" ; // see "-gui" in installer
-				std::string exe = isWindows() ? "emailrelay-gui.exe" : "emailrelay-gui.real" ;
-				f << argv0.basename() << "= %dir-install%/" << sdir << exe << " +x" << std::endl ;
+				G::File::mkdir( pdir , std::nothrow ) ;
+				std::ofstream f ;
+				G::File::open( f , pdir+"payload.cfg" , G::File::Text() ) ;
+				if( isWindows() )
+				{
+					G::File::copyInto( "emailrelay-gui.exe" , pdir , std::nothrow ) ;
+					f << "emailrelay-gui.exe= %dir-install%/emailrelay-gui.exe +x" << std::endl ;
+				}
+				else
+				{
+					G::File::mkdirs( pdir+"usr/sbin" , std::nothrow ) ;
+					G::File::copyInto( argv0 , pdir + "usr/sbin" , std::nothrow ) ;
+					G::File::copyInto( "/usr/sbin/emailrelay" , pdir + "usr/sbin" , std::nothrow ) ;
+					f << "usr/sbin/=%dir-install%/sbin/" << std::endl ;
+				}
+				GPage::setTestMode() ;
 			}
 
 			// look for the payload (for install mode)
@@ -289,17 +328,17 @@ int main( int argc , char * argv [] )
 			G::Path pointer_file ;
 			if( is_mac )
 				pointer_file = search( argv0.dirname() , "dir.cfg" , ".." ) ;
-			if( pointer_file == G::Path() )
+			if( pointer_file.empty() )
 				pointer_file = search( argv0.dirname() , pointerFilename(argv0) , "." ) ;
 
 			// choose install or configure mode
-			bool configure_mode = is_mac || payload_path == G::Path() ;
-			G_LOG_S( "pointer file [" << pointer_file << "]" ) ;
-			G_LOG_S( "payload [" << payload_path << "]" ) ;
-			G_LOG_S( "running in " << (configure_mode?"configure":"install") << " mode" ) ;
+			bool configure_mode = is_mac || payload_path.empty() ;
+			G_LOG_S( "main: pointer file [" << pointer_file << "]" ) ;
+			G_LOG_S( "main: payload [" << payload_path << "]" ) ;
+			G_LOG_S( "main: running in " << (configure_mode?"configure":"install") << " mode" ) ;
 
 			// fail if no payload and no pointer file
-			if( configure_mode && pointer_file == G::Path() )
+			if( configure_mode && pointer_file.empty() )
 				throw std::runtime_error( "cannot find a 'payload' directory for installation or a "
 					"'" + pointerFilename(argv0) + "' pointer file to allow reconfiguration: "
 					"this program has probably been moved away from its original location: "
@@ -311,19 +350,19 @@ int main( int argc , char * argv [] )
 			{
 				if( G::File::exists(pointer_file) )
 				{
-					G_LOG_S( "reading directories from [" << pointer_file << "]" ) ;
+					G_LOG_S( "main: reading directories from [" << pointer_file << "]" ) ;
 					pointer_map = G::MapFile( pointer_file ) ;
 				}
 				else
 				{
-					G_LOG_S( "cannot read install directories from file [" << pointer_file << "]: "
+					G_LOG_S( "main: cannot read install directories from file [" << pointer_file << "]: "
 						"file not found: using built-in defaults" ) ;
 				}
 			}
 
+			// add some handy substitution variables
 			if( is_mac )
 			{
-				// add some handy substitution variables
 				pointer_map.add( "dir-contents" , (argv0.dirname()+"..").collapsed().str() ) ;
 				pointer_map.add( "dir-bundle" , (argv0.dirname()+".."+"..").collapsed().str() ) ;
 			}
@@ -335,7 +374,7 @@ int main( int argc , char * argv [] )
 			G::MapFile server_config_map ;
 			if( configure_mode )
 			{
-				G_LOG_S( "editing configuration file [" << server_config_file << "]" ) ;
+				G_LOG_S( "main: editing configuration file [" << server_config_file << "]" ) ;
 				server_config_map = ServerConfiguration(server_config_file).map() ;
 				if( server_config_map.contains("pid-file") )
 					dir_run = server_config_map.pathValue("pid-file").dirname() ;
@@ -350,7 +389,7 @@ int main( int argc , char * argv [] )
 				{
 					// read the server exe referred to in startup batch file
 					G::Path server_exe = ServerConfiguration::exe( server_config_file ) ;
-					dir_install = server_exe != G::Path() && server_exe.dirname() != G::Path() ?
+					dir_install = !server_exe.empty() && !server_exe.dirname().empty() ?
 						server_exe.dirname() : argv0.dirname() ;
 				}
 				else
@@ -361,9 +400,9 @@ int main( int argc , char * argv [] )
 			}
 
 			// log the directory paths
-			G_LOG_S( "dir-config=" << dir_config ) ;
-			G_LOG_S( "dir-install=" << dir_install ) ;
-			G_LOG_S( "dir-run=" << dir_run ) ;
+			G_LOG_S( "main: dir-config=" << dir_config ) ;
+			G_LOG_S( "main: dir-install=" << dir_install ) ;
+			G_LOG_S( "main: dir-run=" << dir_run ) ;
 
 			// set up the gui pages' config map
 			G::MapFile pages_config = server_config_map ;
@@ -375,11 +414,11 @@ int main( int argc , char * argv [] )
 			// set widget states based on the current file-system state
 			if( configure_mode )
 			{
-				std::string fname = GLink::filename( "E-MailRelay" ) ;
+				std::string fname = G::Link::filename( "E-MailRelay" ) ;
 				pages_config.add( "start-on-boot" , Boot::installed(Dir::boot(),"emailrelay") ? "y" : "n" ) ;
-				pages_config.add( "start-link-desktop" , GLink::exists(Dir::desktop()+fname) ? "y" : "n" ) ;
-				pages_config.add( "start-link-menu" , GLink::exists(Dir::menu()+fname) ? "y" : "n" ) ;
-				pages_config.add( "start-at-login" , GLink::exists(Dir::autostart()+fname) ? "y" : "n" ) ;
+				pages_config.add( "start-link-desktop" , G::Link::exists(Dir::desktop()+fname) ? "y" : "n" ) ;
+				pages_config.add( "start-link-menu" , G::Link::exists(Dir::menu()+fname) ? "y" : "n" ) ;
+				pages_config.add( "start-at-login" , G::Link::exists(Dir::autostart()+fname) ? "y" : "n" ) ;
 			}
 
 			// check the config file (or windows batch file) will be writeable
@@ -388,7 +427,8 @@ int main( int argc , char * argv [] )
 				if( G::File::exists(server_config_file) )
 				{
 					std::string more_help = isWindows() ? " or run as administrator" : "" ;
-					std::ofstream f( server_config_file.str().c_str() , std::ios_base::app ) ; // app for no-truncate
+					std::ofstream f ;
+					G::File::open( f , server_config_file , G::File::Append() ) ;
 					if( !f.good() )
 						throw std::runtime_error( "cannot write [" + server_config_file.str() + "]"
 							": check file permissions" + more_help ) ;
@@ -403,45 +443,44 @@ int main( int argc , char * argv [] )
 				}
 			}
 
-			// default translator
-			QTranslator qt_translator;
-			qt_translator.load(QString("qt_")+QLocale::system().name());
-			app.installTranslator(&qt_translator);
-
-			// application translator
-			QTranslator translator;
-			translator.load(QString("emailrelay_")+QLocale::system().name());
-			app.installTranslator(&translator);
-
 			// create the installer - the ProgressPage extracts dumps from all the
 			// dialog pages and passes the result to the installer
 			Installer installer( !configure_mode , isWindows() , isMac() , payload_path ) ;
+			bool can_generate = installer.canGenerateKey() ;
 
 			// test whether we have run before -- this is normally the same
 			// as the configure-mode flag because the first run is always in
 			// install mode -- but for a mac bundle we are always in configure
 			// mode, so we add a flag file into the distribution bundle that gets
 			// deleted by a successful first configuration (see GDialog)
-			G::Path virgin_flag_file = is_mac ? (dir_run+".new") : G::Path() ;
-			if( virgin_flag_file != G::Path() ) G_LOG_S( "virgin-file=" << virgin_flag_file ) ;
-			bool run_before = virgin_flag_file == G::Path() ? configure_mode : !G::File::exists(virgin_flag_file) ;
+			bool run_before = configure_mode ;
+			G::Path virgin_flag_file ;
+			if( is_mac )
+			{
+				virgin_flag_file = dir_run + ".new" ;
+				G_LOG_S( "main: virgin-file=" << virgin_flag_file ) ;
+				run_before = !G::File::exists( virgin_flag_file ) ;
+			}
 
 			// create the dialog and all its pages
 			const bool licence_accepted = run_before ;
-			GDialog d( false , virgin_flag_file ) ;
-			d.add( new TitlePage(d,pages_config,"title","license","",false,false) ) ;
-			d.add( new LicensePage(d,pages_config,"license","directory","",false,false,licence_accepted) ) ;
-			d.add( new DirectoryPage(d,pages_config,"directory","dowhat","",false,false,!configure_mode,is_mac) ) ;
-			d.add( new DoWhatPage(d,pages_config,"dowhat","pop","smtpserver",false,false) ) ;
-			d.add( new PopPage(d,pages_config,"pop","smtpserver","logging",false,false,configure_mode) ) ;
-			d.add( new SmtpServerPage(d,pages_config,"smtpserver","smtpclient","",false,false,configure_mode) ) ;
-			d.add( new SmtpClientPage(d,pages_config,"smtpclient","filter","",false,false,configure_mode) ) ;
-			d.add( new FilterPage(d,pages_config,"filter","logging","",false,false,isWindows()) ) ;
-			d.add( new LoggingPage(d,pages_config,"logging","listening","",false,false) ) ;
-			d.add( new ListeningPage(d,pages_config,"listening","startup","",false,false) ) ;
-			d.add( new StartupPage(d,pages_config,"startup","ready","",false,false,Boot::able(Dir::boot()),is_mac) ) ;
-			d.add( new ReadyPage(d,pages_config,"ready","progress","",true,false,!configure_mode) ) ;
-			d.add( new ProgressPage(d,pages_config,"progress","","",true,true,installer) ) ;
+			const bool with_launch = !configure_mode ;
+			GDialog d( virgin_flag_file , with_launch ) ;
+			//d.enableHelpButton() ;
+			//d.enableLaunchButton() ;
+			d.add( new TitlePage(d,pages_config,"title","license","") ) ;
+			d.add( new LicensePage(d,pages_config,"license","directory","",licence_accepted) ) ;
+			d.add( new DirectoryPage(d,pages_config,"directory","dowhat","",!configure_mode,is_mac) ) ;
+			d.add( new DoWhatPage(d,pages_config,"dowhat","pop","smtpserver") ) ;
+			d.add( new PopPage(d,pages_config,"pop","smtpserver","logging",configure_mode) ) ;
+			d.add( new SmtpServerPage(d,pages_config,"smtpserver","smtpclient","",configure_mode,can_generate&&!configure_mode) ) ;
+			d.add( new SmtpClientPage(d,pages_config,"smtpclient","filter","",configure_mode) ) ;
+			d.add( new FilterPage(d,pages_config,"filter","logging","",!configure_mode,isWindows()) ) ;
+			d.add( new ListeningPage(d,pages_config,"listening","startup","") ) ;
+			d.add( new LoggingPage(d,pages_config,"logging","listening","") ) ;
+			d.add( new StartupPage(d,pages_config,"startup","ready","",Boot::able(Dir::boot()),is_mac) ) ;
+			d.add( new ReadyPage(d,pages_config,"ready","progress","",!configure_mode) ) ;
+			d.add( new ProgressPage(d,pages_config,"progress","","",installer) ) ;
 			d.add() ;
 
 			// set the dialog dimensions
@@ -457,7 +496,7 @@ int main( int argc , char * argv [] )
 		catch( std::exception & e )
 		{
 			error( e.what() ) ;
-			std::string message = G::Str::wrap( e.what() , "" , "" , 40 ) ;
+			std::string message = G::StringWrap::wrap( e.what() , "" , "" , 40 ) ;
 			qCritical( "exception: %s" , message.c_str() ) ;
 		}
 		catch(...)
@@ -469,7 +508,7 @@ int main( int argc , char * argv [] )
 	}
 	catch( std::exception & e )
 	{
-		std::string message = G::Str::wrap( e.what() , "" , "" , 40 ) ;
+		std::string message = G::StringWrap::wrap( e.what() , "" , "" , 40U ) ;
 		qCritical( "exception: %s" , message.c_str() ) ;
 	}
 	catch(...)
@@ -479,4 +518,3 @@ int main( int argc , char * argv [] )
 	return 1 ;
 }
 
-/// \file guimain.cpp
