@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -24,6 +24,7 @@
 #include "gdef.h"
 #include "garg.h"
 #include "ggetopt.h"
+#include "goptions.h"
 #include "output.h"
 #include "configuration.h"
 #include "output.h"
@@ -35,7 +36,7 @@ namespace Main
 	class CommandLine ;
 }
 
-/// \class Main::CommandLine
+//| \class Main::CommandLine
 /// A class which deals with the command-line interface to the process, both
 /// input from command-line parameters and feedback to (eg.) stdout.
 ///
@@ -44,7 +45,7 @@ namespace Main
 class Main::CommandLine
 {
 public:
-	CommandLine( Main::Output & output , const G::Arg & arg , const std::string & spec ,
+	CommandLine( Main::Output & output , const G::Arg & arg , const G::Options & spec ,
 		const std::string & version ) ;
 			///< Constructor.
 
@@ -57,7 +58,7 @@ public:
 	const G::Options & options() const ;
 		///< Exposes the options sub-object.
 
-	G::Arg::size_type argc() const ;
+	std::size_t argc() const ;
 		///< Returns the number of non-option arguments on the command line.
 
 	bool hasUsageErrors() const ;
@@ -102,9 +103,13 @@ public:
 	void logSemanticWarnings( const G::StringArray & semantic_warnings ) const ;
 		///< Logs the given semantic warnings. See Configuration::semanticWarnings().
 
+public:
+	CommandLine( const CommandLine & ) = delete ;
+	CommandLine( CommandLine && ) = delete ;
+	void operator=( const CommandLine & ) = delete ;
+	void operator=( CommandLine && ) = delete ;
+
 private:
-	CommandLine( const CommandLine & ) g__eq_delete ;
-	void operator=( const CommandLine & ) g__eq_delete ;
 	void showUsage( bool e ) const ;
 	void showShortHelp( bool e ) const ;
 	void showExtraHelp( bool e ) const ;
@@ -119,6 +124,7 @@ private:
 	std::string m_version ;
 	G::Arg m_arg ;
 	G::GetOpt m_getopt ;
+	bool m_verbose ;
 	std::string m_insanity ;
 } ;
 
