@@ -1,19 +1,22 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
+///
+/// \file licence.cpp
+///
 const char * licence[] = {
 "                    GNU GENERAL PUBLIC LICENSE" ,
 "                       Version 3, 29 June 2007" ,
@@ -689,6 +692,5 @@ const char * licence[] = {
 "the library.  If this is what you want to do, use the GNU Lesser General" ,
 "Public License instead of this License.  But first, please read" ,
 "<http://www.gnu.org/philosophy/why-not-lgpl.html>." ,
-0
+nullptr
 } ;
-/// \file licence.cpp
