@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -53,7 +53,7 @@ namespace Main
 	class Run ;
 }
 
-/// \class Main::Run
+//| \class Main::Run
 /// A top-level class for the process.
 /// Usage:
 /// \code
@@ -61,7 +61,7 @@ namespace Main
 /// {
 ///   G::Arg arg( argc , argv ) ;
 ///   Output output ;
-///   Main::Run run( output , arg , Main::Options::spec() ) ;
+///   Main::Run run( output , arg ) ;
 ///   run.configure() ;
 ///   if( run.runnable() )
 ///      run.run() ;
@@ -72,10 +72,10 @@ namespace Main
 class Main::Run : private GNet::EventHandler
 {
 public:
-	Run( Output & output , const G::Arg & arg , const std::string & option_spec , bool has_gui ) ;
+	Run( Output & output , const G::Arg & arg , bool is_windows = false , bool has_gui = false ) ;
 		///< Constructor. Tries not to throw.
 
-	virtual ~Run() ;
+	~Run() override ;
 		///< Destructor.
 
 	void configure() ;
@@ -100,7 +100,7 @@ public:
 	static std::string versionNumber() ;
 		///< Returns the application version number string.
 
-	G::Slot::Signal4<std::string,std::string,std::string,std::string> & signal() ;
+	G::Slot::Signal<std::string,std::string,std::string,std::string> & signal() ;
 		///< Provides a signal which is activated when something changes.
 
 private:
@@ -121,22 +121,26 @@ private:
 		}
 	} ;
 
+public:
+	Run( const Run & ) = delete ;
+	Run( Run && ) = delete ;
+	void operator=( const Run & ) = delete ;
+	void operator=( Run && ) = delete ;
+
 private:
-	Run( const Run & ) g__eq_delete ;
-	void operator=( const Run & ) g__eq_delete ;
 	void doForwardingOnStartup( G::PidFile & ) ;
-	void closeFiles() ;
-	void closeMoreFiles() ;
+	void closeFiles() const ;
+	void closeMoreFiles() const ;
 	void commit( G::PidFile & ) ;
 	std::string smtpIdent() const ;
 	void recordPid() ;
 	const CommandLine & commandline() const ;
-	void onClientDone( std::string ) ; // Client::doneSignal()
-	void onClientEvent( std::string , std::string , std::string ) ; // Client::eventSignal()
-	void onServerEvent( std::string , std::string ) ; // Server::eventSignal()
+	void onClientDone( const std::string & ) ; // Client::doneSignal()
+	void onClientEvent( const std::string & , const std::string & , const std::string & ) ; // Client::eventSignal()
+	void onServerEvent( const std::string & , const std::string & ) ; // Server::eventSignal()
 	void onStoreUpdateEvent() ;
 	void onStoreRescanEvent() ;
-	void onNetworkEvent( std::string , std::string ) ;
+	void onNetworkEvent( const std::string & , const std::string & ) ;
 	void emit( const std::string & , const std::string & , const std::string & = std::string() , const std::string & = std::string() ) ;
 	void onPollTimeout() ;
 	void requestForwarding( const std::string & = std::string() ) ;
@@ -155,10 +159,11 @@ private:
 	void checkScripts() const ;
 	void checkVerifierScript( const std::string & ) const ;
 	void checkFilterScript( const std::string & ) const ;
+	void checkThreading() const ;
 	std::string versionString() const ;
 	static std::string buildConfiguration() ;
 	G::Path appDir() const ;
-	unique_ptr<GSmtp::AdminServer> newAdminServer( GNet::ExceptionSink ,
+	std::unique_ptr<GSmtp::AdminServer> newAdminServer( GNet::ExceptionSink ,
 		const Configuration & , GSmtp::MessageStore & , const GNet::ServerPeerConfig & ,
 		const GSmtp::Client::Config & , const GAuth::Secrets & , const std::string & ) ;
 
@@ -166,27 +171,27 @@ private:
 	Output & m_output ;
 	GNet::ExceptionSink m_es_rethrow ;
 	GNet::ExceptionSink m_es_nothrow ;
-	std::string m_option_spec ;
+	bool m_is_windows ;
 	G::Arg m_arg ;
-	G::Slot::Signal4<std::string,std::string,std::string,std::string> m_signal ;
-	unique_ptr<CommandLine> m_commandline ;
-	unique_ptr<Configuration> m_configuration ;
-	unique_ptr<G::LogOutput> m_log_output ;
-	unique_ptr<GNet::EventLoop> m_event_loop ;
-	unique_ptr<GNet::TimerList> m_timer_list ;
-	unique_ptr<GNet::Timer<Run> > m_forwarding_timer ;
-	unique_ptr<GNet::Timer<Run> > m_poll_timer ;
-	unique_ptr<GNet::Timer<Run> > m_queue_timer ;
-	unique_ptr<GSsl::Library> m_tls_library ;
-	unique_ptr<GNet::Monitor> m_monitor ;
-	unique_ptr<GSmtp::FileStore> m_store ;
-	unique_ptr<GAuth::Secrets> m_client_secrets ;
-	unique_ptr<GAuth::Secrets> m_server_secrets ;
-	unique_ptr<GAuth::Secrets> m_pop_secrets ;
-	unique_ptr<GSmtp::Server> m_smtp_server ;
-	unique_ptr<GPop::Store> m_pop_store ;
-	unique_ptr<GPop::Server> m_pop_server ;
-	unique_ptr<GSmtp::AdminServer> m_admin_server ;
+	G::Slot::Signal<std::string,std::string,std::string,std::string> m_signal ;
+	std::unique_ptr<CommandLine> m_commandline ;
+	std::unique_ptr<Configuration> m_configuration ;
+	std::unique_ptr<G::LogOutput> m_log_output ;
+	std::unique_ptr<GNet::EventLoop> m_event_loop ;
+	std::unique_ptr<GNet::TimerList> m_timer_list ;
+	std::unique_ptr<GNet::Timer<Run> > m_forwarding_timer ;
+	std::unique_ptr<GNet::Timer<Run> > m_poll_timer ;
+	std::unique_ptr<GNet::Timer<Run> > m_queue_timer ;
+	std::unique_ptr<GSsl::Library> m_tls_library ;
+	std::unique_ptr<GNet::Monitor> m_monitor ;
+	std::unique_ptr<GSmtp::FileStore> m_store ;
+	std::unique_ptr<GAuth::Secrets> m_client_secrets ;
+	std::unique_ptr<GAuth::Secrets> m_server_secrets ;
+	std::unique_ptr<GAuth::Secrets> m_pop_secrets ;
+	std::unique_ptr<GSmtp::Server> m_smtp_server ;
+	std::unique_ptr<GPop::Store> m_pop_store ;
+	std::unique_ptr<GPop::Server> m_pop_server ;
+	std::unique_ptr<GSmtp::AdminServer> m_admin_server ;
 	GNet::ClientPtr<GSmtp::Client> m_client_ptr ;
 	std::deque<QueueItem> m_queue ;
 	std::string m_forwarding_reason ;
@@ -198,7 +203,7 @@ private:
 inline
 const Main::Configuration & Main::Run::configuration() const
 {
-	return *m_configuration.get() ;
+	return *m_configuration ;
 }
 
 #endif
