@@ -1,22 +1,22 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
-//
-// winapp.cpp
-//
+///
+/// \file winapp.cpp
+///
 
 #include "gdef.h"
 #include "winapp.h"
@@ -28,22 +28,51 @@
 #include "gcontrol.h"
 #include "gdialog.h"
 #include "resource.h"
-#if G_MINGW
-static int cfg_tab_stop = 200 ; // edit control tab stops are flakey in mingw
-#else
-static int cfg_tab_stop = 120 ;
-#endif
 
 namespace Main
 {
 	class Box ;
 	class Config ;
+	struct PixelLayout
+	{
+		explicit PixelLayout( bool verbose ) ;
+		int tabstop () const { return m_tabstop ; }
+		unsigned int width() const { return m_width ; }
+		unsigned int width2() const { return m_width2 ; }
+		private:
+		static bool isWine() ;
+		int m_tabstop ;
+		unsigned int m_width ;
+		unsigned int m_width2 ;
+	} ;
 } ;
 
+Main::PixelLayout::PixelLayout( bool verbose )
+{
+	if( isWine() )
+	{
+		m_tabstop = verbose ? 37 : 28 ;
+		m_width = verbose ? 60U : 85U ;
+		m_width2 = verbose ? 48U : 65U ;
+	}
+	else
+	{
+		m_tabstop = verbose ? 120 : 90 ;
+		m_width = verbose ? 60U : 80U ;
+		m_width2 = verbose ? 48U : 80U ;
+	}
+}
+
+bool Main::PixelLayout::isWine()
+{
+	HMODULE h = GetModuleHandle( "ntdll.dll" ) ;
+	return h && !!GetProcAddress( h , "wine_get_version" ) ;
+}
+
 class Main::Box : public GGui::Dialog
 {
 public:
-	Box( GGui::ApplicationBase & app , const G::StringArray & text ) ;
+	Box( GGui::ApplicationBase & app , const G::StringArray & text , int tabstop ) ;
 	bool run() ;
 
 private: // overrides
@@ -52,6 +81,7 @@ private: // overrides
 private:
 	GGui::EditBox m_edit ;
 	G::StringArray m_text ;
+	int m_tabstop ;
 } ;
 
 // ==
@@ -77,13 +107,16 @@ void Main::WinApp::disableOutput()
 
 void Main::WinApp::init( const Configuration & cfg )
 {
-	m_form_cfg.reset( new Configuration(cfg) ) ;
+	m_form_cfg = std::make_unique<Configuration>( cfg ) ;
 	m_cfg = Config::create( cfg ) ;
 }
 
 int Main::WinApp::exitCode() const
 {
-	if( G::Test::enabled("special-exit-code") ) return (G::threading::works()?23:25) ;
+	// see test/Server.pm hasDebug()
+	if( G::Test::enabled("special-exit-code") )
+		return (G::threading::works()?23:25) ;
+
 	return m_exit_code ;
 }
 
@@ -109,7 +142,7 @@ bool Main::WinApp::onCreate()
 	{
 		try
 		{
-			m_tray.reset( new GGui::Tray(resource(),*this,"E-MailRelay") ) ;
+			m_tray = std::make_unique<GGui::Tray>( resource() , *this , "E-MailRelay" ) ;
 		}
 		catch( std::exception & e )
 		{
@@ -123,13 +156,13 @@ bool Main::WinApp::onCreate()
 	return true ;
 }
 
-namespace
+namespace Main
 {
 	struct ScopeFinalReset
 	{
-		ScopeFinalReset( unique_ptr<Main::WinMenu> & ptr ) : m_ptr(ptr) {}
+		ScopeFinalReset( std::unique_ptr<Main::WinMenu> & ptr ) : m_ptr(ptr) {}
 		~ScopeFinalReset() { m_ptr.reset() ; }
-		unique_ptr<Main::WinMenu> & m_ptr ;
+		std::unique_ptr<Main::WinMenu> & m_ptr ;
 	} ;
 }
 
@@ -141,7 +174,7 @@ void Main::WinApp::onTrayRightMouseButtonDown()
 	// other event notifications before then, so make the menu
 	// accessible to them via a data member)
 	ScopeFinalReset final_reset( m_menu ) ;
-	m_menu.reset( new WinMenu(IDR_MENU1) ) ;
+	m_menu = std::make_unique<WinMenu>( IDR_MENU1 ) ;
 
 	bool form_is_visible = m_form.get() != nullptr && m_form.get()->visible() ;
 	bool with_open = !form_is_visible ;
@@ -188,7 +221,7 @@ LRESULT Main::WinApp::onUserOther( WPARAM wparam , LPARAM )
 	return 0L ;
 }
 
-namespace
+namespace Main
 {
 	struct ScopeSet
 	{
@@ -232,9 +265,9 @@ void Main::WinApp::doOpen()
 		bool form_with_icon = true ;
 		bool form_with_system_menu_quit = m_cfg.with_sysmenu_quit ;
 
-		m_form.reset( new WinForm( hinstance() , *m_form_cfg.get() ,
+		m_form = std::make_unique<WinForm>( hinstance() , *m_form_cfg.get() ,
 			form_hparent , handle() , form_style , form_allow_apply ,
-			form_with_icon , form_with_system_menu_quit ) ) ;
+			form_with_icon , form_with_system_menu_quit ) ;
 	}
 
 	if( m_cfg.restore_on_open )
@@ -296,29 +329,34 @@ void Main::WinApp::onWindowException( std::exception & e )
 	GGui::Window::onWindowException( e ) ;
 }
 
-G::Options::Layout Main::WinApp::layout() const
+G::Options::Layout Main::WinApp::outputLayout( bool verbose ) const
 {
-	G::Options::Layout layout( 0U ) ;
+	G::Options::Layout layout ;
 	layout.separator = "\t" ;
-	layout.indent = "\t\t" ;
-	layout.width = 100U ;
+	//layout.column
+	layout.width = PixelLayout(verbose).width() ;
+	layout.width2 = PixelLayout(verbose).width2() ;
+	layout.margin = 0U ;
+	//layout.level
+	//layout.extra
+	//layout.usage_other
 	return layout ;
 }
 
-bool Main::WinApp::simpleOutput() const
+bool Main::WinApp::outputSimple() const
 {
 	return false ;
 }
 
-void Main::WinApp::output( const std::string & text , bool )
+void Main::WinApp::output( const std::string & text , bool , bool verbose )
 {
 	if( !m_disable_output )
 	{
 		G::StringArray text_lines ;
 		G::Str::splitIntoFields( text , text_lines , "\r\n" ) ;
-		if( text_lines.size() > 25U ) // eg. "--help"
+		if( text_lines.size() > 10U ) // eg. "--help"
 		{
-			Box box( *this , text_lines ) ;
+			Box box( *this , text_lines , PixelLayout(verbose).tabstop() ) ;
 			if( ! box.run() )
 				messageBox( text ) ;
 		}
@@ -332,16 +370,17 @@ void Main::WinApp::output( const std::string & text , bool )
 void Main::WinApp::onError( const std::string & text )
 {
 	// called from WinMain(), possibly before init()
-	output( text , true ) ; // override implemented above
+	output( text , true , false ) ; // override implemented above
 	m_exit_code = 1 ;
 }
 
 // ==
 
-Main::Box::Box( GGui::ApplicationBase & app , const G::StringArray & text ) :
+Main::Box::Box( GGui::ApplicationBase & app , const G::StringArray & text , int tabstop ) :
 	GGui::Dialog(app,false) ,
 	m_edit(*this,IDC_EDIT1) ,
-	m_text(text)
+	m_text(text) ,
+	m_tabstop(tabstop)
 {
 }
 
@@ -350,8 +389,7 @@ bool Main::Box::onInit()
 	G_DEBUG( "Main::Box::onInit" ) ;
 
 	std::vector<int> tabs ;
-	tabs.push_back( 10 ) ;
-	tabs.push_back( cfg_tab_stop ) ;
+	tabs.push_back( m_tabstop ) ;
 	m_edit.setTabStops( tabs ) ;
 
 	m_edit.set( m_text ) ;
@@ -461,4 +499,3 @@ Main::WinApp::Config Main::WinApp::Config::create( const Main::Configuration & c
 	}
 }
 
-/// \file winapp.cpp
