@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -31,7 +31,7 @@ namespace GGui
 	class Tray ;
 } ;
 
-/// \class GGui::Tray
+//| \class GGui::Tray
 /// Manages an icon within the system tray.
 ///
 class GGui::Tray
@@ -52,9 +52,11 @@ public:
 	~Tray() ;
 		///< Destructor. Removes the icon from the system tray.
 
-private:
-	void operator=( const Tray & ) g__eq_delete ;
-	Tray( const Tray & ) g__eq_delete ;
+public:
+	void operator=( const Tray & ) = delete ;
+	void operator=( Tray && ) = delete ;
+	Tray( const Tray & ) = delete ;
+	Tray( Tray && ) = delete ;
 
 private:
 	NOTIFYICONDATAA m_info ;
