@@ -1,22 +1,22 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
-//
-// gwindow.cpp
-//
+///
+/// \file gwindow.cpp
+///
 
 #include "gdef.h"
 #include "gwindow.h"
@@ -36,7 +36,7 @@ GGui::Window::Window( HWND hwnd ) :
 GGui::Window::~Window()
 {
 	if( handle() )
-		::SetWindowLongPtr( handle() , 0 , 0 ) ;
+		SetWindowLongPtr( handle() , 0 , 0 ) ;
 }
 
 bool GGui::Window::registerWindowClass( const std::string & class_name_in ,
@@ -63,7 +63,7 @@ bool GGui::Window::registerWindowClass( const std::string & class_name_in ,
 	c.lpszMenuName = menu_name ;
 	c.lpszClassName = class_name.c_str() ;
 
-	return ::RegisterClass( &c ) != 0 ;
+	return RegisterClass( &c ) != 0 ;
 }
 
 bool GGui::Window::create( const std::string & class_name ,
@@ -71,42 +71,42 @@ bool GGui::Window::create( const std::string & class_name ,
 	int x , int y , int dx , int dy ,
 	HWND parent , HMENU menu , HINSTANCE hinstance )
 {
-	G_ASSERT( handle() == NULL ) ;
+	G_ASSERT( handle() == HNULL ) ;
 	G_DEBUG( "GGui::Window::create: \"" << class_name << "\", \"" << title << "\"" ) ;
 
 	DWORD style = style_pair.first ;
 	DWORD extended_style = style_pair.second ;
 
 	void *vp = reinterpret_cast<void*>(this) ;
-	HWND hwnd = ::CreateWindowExA( extended_style , class_name.c_str() , title.c_str() ,
+	HWND hwnd = CreateWindowExA( extended_style , class_name.c_str() , title.c_str() ,
 		style , x , y , dx , dy , parent , menu , hinstance , vp ) ;
 	setHandle( hwnd ) ; // GGui::WindowBase
 
 	G_DEBUG( "GGui::Window::create: handle " << handle() ) ;
-	return handle() != NULL ;
+	return handle() != HNULL ;
 }
 
 void GGui::Window::update()
 {
-	G_ASSERT( handle() != NULL ) ;
-	::UpdateWindow( handle() ) ;
+	G_ASSERT( handle() != HNULL ) ;
+	UpdateWindow( handle() ) ;
 }
 
 void GGui::Window::show( int style )
 {
-	G_ASSERT( handle() != NULL ) ;
-	::ShowWindow( handle() , style ) ;
+	G_ASSERT( handle() != HNULL ) ;
+	ShowWindow( handle() , style ) ;
 }
 
 void GGui::Window::invalidate( bool erase )
 {
-	::InvalidateRect( handle() , NULL , erase ) ;
+	InvalidateRect( handle() , nullptr , erase ) ;
 }
 
 GGui::Window * GGui::Window::instance( HWND hwnd )
 {
-	G_ASSERT( hwnd != NULL ) ;
-	LONG_PTR wl = ::GetWindowLongPtr( hwnd , 0 ) ;
+	G_ASSERT( hwnd != HNULL ) ;
+	LONG_PTR wl = GetWindowLongPtr( hwnd , 0 ) ;
 	void * vp = reinterpret_cast<void*>(wl) ;
 	return reinterpret_cast<Window*>(vp) ;
 }
@@ -120,7 +120,13 @@ extern "C" LRESULT CALLBACK gwindow_wndproc_export( HWND hwnd , UINT message , W
 	catch( std::exception & e )
 	{
 		// never gets here
-		G_DEBUG( "gwindow_wndproc_export: exception absorbed: " << e.what() ) ; G_IGNORE_VARIABLE(std::exception&,e) ;
+		GDEF_IGNORE_PARAM( e ) ;
+		G_DEBUG( "gwindow_wndproc_export: exception absorbed: " << e.what() ) ;
+		return 0 ;
+	}
+	catch(...) // callback
+	{
+		// never gets here
 		return 0 ;
 	}
 }
@@ -174,7 +180,7 @@ LRESULT GGui::Window::wndProc( HWND hwnd , UINT msg , WPARAM wparam , LPARAM lpa
 
 GGui::Window * GGui::Window::instance( CREATESTRUCT * cs )
 {
-	G_ASSERT( cs != NULL ) ;
+	G_ASSERT( cs != nullptr ) ;
 	return reinterpret_cast<Window*>(cs->lpCreateParams) ;
 }
 
@@ -183,30 +189,30 @@ LRESULT GGui::Window::wndProcCore( Window * window , HWND hwnd , UINT msg , WPAR
 	LRESULT result = 0 ;
 	if( msg == WM_CREATE )
 	{
-		G_ASSERT( window != NULL ) ;
+		G_ASSERT( window != nullptr ) ;
 		G_DEBUG( "GGui::Window::wndProc: WM_CREATE: ptr " << window ) ;
 		G_DEBUG( "GGui::Window::wndProc: WM_CREATE: hwnd " << hwnd ) ;
 		void * vp = reinterpret_cast<void*>(window) ;
 		LONG_PTR wl = reinterpret_cast<LONG_PTR>(vp) ;
-		::SetWindowLongPtr( hwnd , 0 , wl ) ;
+		SetWindowLongPtr( hwnd , 0 , wl ) ;
 		window->setHandle( hwnd ) ;
 		result = window->onCreate() ? 0 : -1 ;
 	}
 	else
 	{
-		bool call_default = window == NULL ;
-		if( window != NULL )
+		bool call_default = window == nullptr ;
+		if( window != nullptr )
 			result = window->crack( msg , wparam , lparam , call_default ) ;
 		if( call_default )
-			result = ::DefWindowProc( hwnd , msg , wparam , lparam ) ;
+			result = DefWindowProc( hwnd , msg , wparam , lparam ) ;
 	}
 	return result ;
 }
 
 LRESULT GGui::Window::sendUserString( HWND hwnd , const char * string )
 {
-	G_ASSERT( string != NULL ) ;
-	return ::SendMessage( hwnd , Cracker::wm_user_other() , 0 , reinterpret_cast<LPARAM>(string) ) ;
+	G_ASSERT( string != nullptr ) ;
+	return SendMessage( hwnd , Cracker::wm_user_other() , 0 , reinterpret_cast<LPARAM>(string) ) ;
 }
 
 LRESULT GGui::Window::onUserOther( WPARAM , LPARAM lparam )
@@ -221,26 +227,29 @@ LRESULT GGui::Window::onUserString( const char * )
 
 void GGui::Window::destroy()
 {
-	::DestroyWindow( handle() ) ;
+	DestroyWindow( handle() ) ;
 }
 
-namespace
+namespace GGui
 {
-	std::pair<DWORD,DWORD> make_style( DWORD first , DWORD second = 0 )
+	namespace WindowImp
 	{
-		return std::make_pair( first , second ) ;
+		std::pair<DWORD,DWORD> make_style( DWORD first , DWORD second = 0 )
+		{
+			return std::make_pair( first , second ) ;
+		}
 	}
 }
 
 std::pair<DWORD,DWORD> GGui::Window::windowStyleMain()
 {
-	return make_style( WS_OVERLAPPEDWINDOW ) ;
+	return WindowImp::make_style( WS_OVERLAPPEDWINDOW ) ;
 }
 
 std::pair<DWORD,DWORD> GGui::Window::windowStylePopup()
 {
 	return
-		make_style(
+		WindowImp::make_style(
 			WS_THICKFRAME |
 			WS_POPUP |
 			WS_SYSMENU |
@@ -251,12 +260,12 @@ std::pair<DWORD,DWORD> GGui::Window::windowStylePopup()
 
 std::pair<DWORD,DWORD> GGui::Window::windowStyleChild()
 {
-	return make_style( WS_CHILDWINDOW ) ;
+	return WindowImp::make_style( WS_CHILDWINDOW ) ;
 }
 
 std::pair<DWORD,DWORD> GGui::Window::windowStylePopupNoButton()
 {
-	return make_style( WS_POPUP , WS_EX_TOOLWINDOW ) ;
+	return WindowImp::make_style( WS_POPUP , WS_EX_TOOLWINDOW ) ;
 }
 
 UINT GGui::Window::classStyle( bool redraw )
@@ -274,12 +283,12 @@ HBRUSH GGui::Window::classBrush()
 
 HICON GGui::Window::classIcon()
 {
-	return ::LoadIcon( NULL , IDI_APPLICATION ) ;
+	return LoadIcon( HNULL , IDI_APPLICATION ) ;
 }
 
 HCURSOR GGui::Window::classCursor()
 {
-	return ::LoadCursor( NULL , IDC_ARROW ) ;
+	return LoadCursor( HNULL , IDC_ARROW ) ;
 }
 
 GGui::Size GGui::Window::borderSize( bool has_menu )
@@ -292,11 +301,11 @@ GGui::Size GGui::Window::borderSize( bool has_menu )
 	// MSDN 4 "Ask Dr GUI #10".
 
 	Size size ;
-	size.dx = ::GetSystemMetrics( SM_CXFRAME ) * 2 ;
-	size.dy = ::GetSystemMetrics( SM_CYFRAME ) * 2 ;
-	size.dy += ::GetSystemMetrics( SM_CYCAPTION ) - ::GetSystemMetrics( SM_CYBORDER ) ;
+	size.dx = GetSystemMetrics( SM_CXFRAME ) * 2 ;
+	size.dy = GetSystemMetrics( SM_CYFRAME ) * 2 ;
+	size.dy += GetSystemMetrics( SM_CYCAPTION ) - GetSystemMetrics( SM_CYBORDER ) ;
 	if( has_menu )
-		size.dy += ::GetSystemMetrics( SM_CYMENU ) + ::GetSystemMetrics( SM_CYBORDER ) ;
+		size.dy += GetSystemMetrics( SM_CYMENU ) + GetSystemMetrics( SM_CYBORDER ) ;
 	return size ;
 }
 
@@ -308,9 +317,9 @@ void GGui::Window::resize( Size new_size , bool repaint )
 	// the parent window for child windows
 
 	RECT rect ;
-	if( ::GetWindowRect( handle() , &rect ) )
+	if( GetWindowRect( handle() , &rect ) )
 	{
-		HWND parent = ::GetParent( handle() ) ;
+		HWND parent = GetParent( handle() ) ;
 		const bool child_window = parent != 0 ;
 		if( child_window )
 		{
@@ -318,7 +327,7 @@ void GGui::Window::resize( Size new_size , bool repaint )
 			rect.top = 0 ;
 		}
 
-		::MoveWindow( handle() , rect.left , rect.top , new_size.dx , new_size.dy , repaint ) ;
+		MoveWindow( handle() , rect.left , rect.top , new_size.dx , new_size.dy , repaint ) ;
 	}
 }
 
@@ -329,4 +338,3 @@ void GGui::Window::onWindowException( std::exception & e )
 	Pump::quit( e.what() ) ;
 }
 
-/// \file gwindow.cpp
