@@ -1,16 +1,16 @@
 //
-// Copyright (C) 2001-2019 Graeme Walker <graeme_walker@users.sourceforge.net>
-//
+// Copyright (C) 2001-2021 Graeme Walker <graeme_walker@users.sourceforge.net>
+// 
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation, either version 3 of the License, or
 // (at your option) any later version.
-//
+// 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
-//
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program.  If not, see <http://www.gnu.org/licenses/>.
 // ===
@@ -18,8 +18,8 @@
 /// \file gwindow.h
 ///
 
-#ifndef G_WINDOW_H
-#define G_WINDOW_H
+#ifndef G_GUI_WINDOW_H
+#define G_GUI_WINDOW_H
 
 #include "gdef.h"
 #include "gcracker.h"
@@ -31,7 +31,7 @@ namespace GGui
 	class Window ;
 }
 
-/// \class GGui::Window
+//| \class GGui::Window
 /// A window class. Window messages should be processed by overriding
 /// the 'onWhatever()' virtual functions inherited from GGui::Cracker.
 ///
@@ -162,7 +162,7 @@ public:
 		///< Returns a value for create() window_style for a typical
 		///< 'main' window.
 		///<
-		///< The create() parent window parameter should be NULL, and
+		///< The create() parent window parameter should be null, and
 		///< the x,y,dx,dy parameters will normally be CW_USEDEFAULT.
 
 	static std::pair<DWORD,DWORD> windowStylePopup() ;
@@ -172,15 +172,15 @@ public:
 		///< activated, has a title bar but no minimise/maximise buttons,
 		///< and stays on top of its parent (if any).
 		///<
-		///< The create() parent window parameter may be NULL -- if
-		///< NULL then this window will be independently iconised with
+		///< The create() parent window parameter may be null -- if
+		///< null then this window will be independently iconised with
 		///< a separate button on the toolbar.
 
 	static std::pair<DWORD,DWORD> windowStyleChild() ;
 		///< Returns a value for create() window_style for a typical
 		///< 'child' window.
 		///<
-		///< The create() parent window parameter cannot be NULL.
+		///< The create() parent window parameter cannot be null.
 
 	static std::pair<DWORD,DWORD> windowStylePopupNoButton() ;
 		///< Returns a value for create() window_style for a non-visible
@@ -219,9 +219,13 @@ protected:
 		///< window procedure. The default implementation
 		///< posts a quit message.
 
+public:
+	Window( const Window & ) = delete ;
+	Window( Window && ) = delete ;
+	void operator=( const Window & ) = delete ;
+	void operator=( Window && ) = delete ;
+
 private:
-	Window( const Window & other ) g__eq_delete ;
-	void operator=( const Window & other ) g__eq_delete ;
 	static LRESULT wndProcCore( Window * , HWND , UINT , WPARAM , LPARAM ) ;
 	virtual LRESULT onUserOther( WPARAM , LPARAM ) ;
 	static Window * instance( CREATESTRUCT * ) ;
